# coding=utf-8
"""
Permission logic module
"""
from permission.logics.base import PermissionLogic
from permission.logics.author import AuthorPermissionLogic
from permission.logics.collaborators import CollaboratorsPermissionLogic
from permission.logics.groupin import GroupInPermissionLogic
from permission.logics.oneself import OneselfPermissionLogic
from permission.logics.staff import StaffPermissionLogic



# coding=utf-8
"""
Permission logic module for author based permission system
"""
from permission.conf import settings
from permission.logics.base import PermissionLogic
from permission.compat import is_authenticated


class StaffPermissionLogic(PermissionLogic):
    """
    Permission logic class for is_staff authority based permission system
    """
    def __init__(self,
                 any_permission=None,
                 add_permission=None,
                 change_permission=None,
                 delete_permission=None):
        """
        Constructor

        Parameters
        ----------
        any_permission : boolean
            True for give any permission of the specified object to the staff
            user. Default value will be taken from
            ``PERMISSION_DEFAULT_SPL_ANY_PERMISSION`` in
            settings.
        add_permission : boolean
            True for give change permission of the specified object to the
            staff user.
            It will be ignored if :attr:`any_permission` is True.
            Default value will be taken from
            ``PERMISSION_DEFAULT_SPL_ADD_PERMISSION`` in
            settings.
        change_permission : boolean
            True for give change permission of the specified object to the
            staff user.
            It will be ignored if :attr:`any_permission` is True.
            Default value will be taken from
            ``PERMISSION_DEFAULT_SPL_CHANGE_PERMISSION`` in
            settings.
        delete_permission : boolean
            True for give delete permission of the specified object to the
            staff user.
            It will be ignored if :attr:`any_permission` is True.
            Default value will be taken from
            ``PERMISSION_DEFAULT_SPL_DELETE_PERMISSION`` in
            settings.
        """
        self.any_permission = any_permission
        self.add_permission = add_permission
        self.change_permission = change_permission
        self.delete_permission = delete_permission

        if self.any_permission is None:
            self.any_permission = \
                settings.PERMISSION_DEFAULT_SPL_ANY_PERMISSION
        if self.add_permission is None:
            self.add_permission = \
                settings.PERMISSION_DEFAULT_SPL_ADD_PERMISSION
        if self.change_permission is None:
            self.change_permission = \
                settings.PERMISSION_DEFAULT_SPL_CHANGE_PERMISSION
        if self.delete_permission is None:
            self.delete_permission = \
                settings.PERMISSION_DEFAULT_SPL_DELETE_PERMISSION

    def has_perm(self, user_obj, perm, obj=None):
        """
        Check if user have permission (of object)

        If the user_obj is not authenticated, it return ``False``.

        If no object is specified, it return ``True`` when the corresponding
        permission was specified to ``True`` (changed from v0.7.0).
        This behavior is based on the django system.
        https://code.djangoproject.com/wiki/RowLevelPermissions

        If an object is specified, it will return ``True`` if the user is
        staff. The staff can add, change or delete the object (you can change
        this behavior to set ``any_permission``, ``add_permission``,
        ``change_permission``, or ``delete_permission`` attributes of this
        instance).

        Parameters
        ----------
        user_obj : django user model instance
            A django user model instance which be checked
        perm : string
            `app_label.codename` formatted permission string
        obj : None or django model instance
            None or django model instance for object permission

        Returns
        -------
        boolean
            Weather the specified user have specified permission (of specified
            object).
        """
        if not is_authenticated(user_obj):
            return False
        # construct the permission full name
        add_permission = self.get_full_permission_string('add')
        change_permission = self.get_full_permission_string('change')
        delete_permission = self.get_full_permission_string('delete')
        if obj is None:
            if user_obj.is_staff:
                if self.add_permission and perm == add_permission:
                    return True
                if self.change_permission and perm == change_permission:
                    return True
                if self.delete_permission and perm == delete_permission:
                    return True
                return self.any_permission
            return False
        elif user_obj.is_active:
            if user_obj.is_staff:
                if self.any_permission:
                    # have any kind of permissions to the obj
                    return True
                if (self.add_permission and
                        perm == add_permission):
                    return True
                if (self.change_permission and
                        perm == change_permission):
                    return True
                if (self.delete_permission and
                        perm == delete_permission):
                    return True
        return False



# coding=utf-8
"""
A utilities of permission handler
"""
from __future__ import unicode_literals
import inspect
from django.core.exceptions import ImproperlyConfigured
from permission.conf import settings
from permission.compat import isstr
from permission.compat import import_string


class PermissionHandlerRegistry(object):
    """
    A registry class of permission handler
    """
    def __init__(self):
        self._registry = {}

    def register(self, model, handler=None):
        """
        Register a permission handler to the model

        Parameters
        ----------
        model : django model class
            A django model class
        handler : permission handler class, string, or None
            A permission handler class or a dotted path

        Raises
        ------
        ImproperlyConfigured
            Raise when the model is abstract model
        KeyError
            Raise when the model is already registered in registry
            The model cannot have more than one handler.
        """
        from permission.handlers import PermissionHandler
        if model._meta.abstract:
            raise ImproperlyConfigured(
                    'The model %s is abstract, so it cannot be registered '
                    'with permission.' % model)
        if model in self._registry:
            raise KeyError("A permission handler class is already "
                            "registered for '%s'" % model)
        if handler is None:
            handler = settings.PERMISSION_DEFAULT_PERMISSION_HANDLER
        if isstr(handler):
            handler = import_string(handler)
        if not inspect.isclass(handler):
            raise AttributeError(
                    "`handler` attribute must be a class. "
                    "An instance was specified.")
        if not issubclass(handler, PermissionHandler):
            raise AttributeError(
                    "`handler` attribute must be a subclass of "
                    "`permission.handlers.PermissionHandler`")

        # Instantiate the handler to save in the registry
        instance = handler(model)
        self._registry[model] = instance

    def unregister(self, model):
        """
        Unregister a permission handler from the model

        Parameters
        ----------
        model : django model class
            A django model class

        Raises
        ------
        KeyError
            Raise when the model have not registered in registry yet.
        """
        if model not in self._registry:
            raise KeyError("A permission handler class have not been "
                           "registered for '%s' yet" % model)
        # remove from registry
        del self._registry[model]

    def get_handlers(self):
        """
        Get registered handler instances

        Returns
        -------
        tuple
            permission handler tuple
        """
        return tuple(self._registry.values())

registry = PermissionHandlerRegistry()
"""Permission handler registry instance"""



# coding=utf-8
from django.test import TestCase, override_settings
from django.template import Context, Template
from ...utils.handlers import registry
from ...templatetags.permissionif import replace_builtin_if
from ..utils import create_user, create_article, create_permission


@override_settings(
    AUTHENTICATION_BACKENDS=(
        'django.contrib.auth.backends.ModelBackend',
        'permission.backends.PermissionBackend',
    ),
    PERMISSION_REPLACE_BUILTIN_IF=False,
)
class PermissionTemplateTagsTestCase(TestCase):

    def setUp(self):
        # store original registry
        self._original_registry = registry._registry
        # clear registry and register mock handler
        registry._registry = {}
        # Make sure builtin if is not replaced
        replace_builtin_if(False)

    def tearDown(self):
        # restore original registry
        registry._registry = self._original_registry

    def test_permissionif_tag(self):
        user = create_user('permission_templatetag_test_user1')
        perm = create_permission('permission_templatetag_test_perm1')

        user.user_permissions.add(perm)

        self.assertTrue(user.has_perm(
            'permission.permission_templatetag_test_perm1'))

        context = Context({
            'user': user,
        })

        out = Template(
            "{% load permissionif %}"
            "{% permission user has "
            "'permission.permission_templatetag_test_perm1' %}"
            "Success"
            "{% else %}"
            "Fail"
            "{% endpermission %}"
        ).render(context)

        self.assertEqual(out, "Success")

    def test_permissionif_tag_elif(self):
        user = create_user('permission_templatetag_test_user1')
        perm = create_permission('permission_templatetag_test_perm1')

        user.user_permissions.add(perm)

        self.assertTrue(user.has_perm(
            'permission.permission_templatetag_test_perm1'))

        context = Context({
            'user': user,
        })

        out = Template(
            "{% load permissionif %}"
            "{% permission user has 'permission.unknown_permission' %}"
            "Fail"
            "{% elpermission user has 'permission.unknown_permisson2' %}"
            "Fail"
            "{% elpermission user has "
            "'permission.permission_templatetag_test_perm1' %}"
            "Success"
            "{% else %}"
            "Fail"
            "{% endpermission %}"
        ).render(context)

        self.assertEqual(out, "Success")

    def test_permissionif_tag_else(self):
        user = create_user('permission_templatetag_test_user1')
        perm = create_permission('permission_templatetag_test_perm1')

        user.user_permissions.add(perm)

        self.assertTrue(
            user.has_perm('permission.permission_templatetag_test_perm1'))

        context = Context({
            'user': user,
        })

        out = Template(
            "{% load permissionif %}"
            "{% permission user has 'permission.unknown_permission' %}"
            "Fail"
            "{% else %}"
            "Success"
            "{% endpermission %}"
        ).render(context)

        self.assertEqual(out, "Success")

    def test_permissionif_tag_with_obj(self):
        from permission.tests.models import Article
        from permission.handlers import PermissionHandler

        user = create_user('permission_templatetag_test_user1')
        art1 = create_article('permission_templatetag_test_article1')
        art2 = create_article('permission_templatetag_test_article2')
        create_permission('permission_templatetag_test_perm1')

        class ArticlePermissionHandler(PermissionHandler):

            def has_perm(self, user_obj, perm, obj=None):
                if perm == 'permission.permission_templatetag_test_perm1':
                    if (obj and obj.title ==
                            'permission_templatetag_test_article2'):
                        return True
                return False
        registry.register(Article, ArticlePermissionHandler)

        self.assertFalse(
            user.has_perm('permission.permission_templatetag_test_perm1'))
        self.assertFalse(
            user.has_perm('permission.permission_templatetag_test_perm1',
                          art1))
        self.assertTrue(
            user.has_perm('permission.permission_templatetag_test_perm1',
                          art2))

        context = Context({
            'user': user,
            'art1': art1,
            'art2': art2,
        })

        out = Template(
            "{% load permissionif %}"
            "{% permission user has "
            "'permission.permission_templatetag_test_perm1' %}"
            "Fail"
            "{% elpermission user has "
            "'permission.permission_templatetag_test_perm1' of art1 %}"
            "Fail"
            "{% elpermission user has "
            "'permission.permission_templatetag_test_perm1' of art2 %}"
            "Success"
            "{% else %}"
            "Fail"
            "{% endpermission %}"
        ).render(context)

        self.assertEqual(out, "Success")

    def test_permissionif_tag_and(self):
        user = create_user('permission_templatetag_test_user1')
        perm1 = create_permission('permission_templatetag_test_perm1')
        perm2 = create_permission('permission_templatetag_test_perm2')

        user.user_permissions.add(perm1, perm2)

        self.assertTrue(
            user.has_perm('permission.permission_templatetag_test_perm1'))
        self.assertTrue(
            user.has_perm('permission.permission_templatetag_test_perm2'))

        context = Context({
            'user': user,
        })

        out = Template(
            "{% load permissionif %}"
            "{% permission user has 'permission.unknown_perm' "
            "and user has 'permission.permission_templatetag_test_perm2' %}"
            "Fail"
            "{% elpermission user has "
            "'permission.permission_templatetag_test_perm1' "
            "and user has 'permission.unknown_perm' %}"
            "Fail"
            "{% elpermission user has "
            "'permission.permission_templatetag_test_perm1' "
            "and user has 'permission.permission_templatetag_test_perm2' %}"
            "Success"
            "{% endpermission %}"
        ).render(context)

        self.assertEqual(out, "Success")

    def test_permissionif_tag_or(self):

        user = create_user('permission_templatetag_test_user1')
        perm1 = create_permission('permission_templatetag_test_perm1')
        create_permission('permission_templatetag_test_perm2')

        user.user_permissions.add(perm1)

        self.assertTrue(
            user.has_perm('permission.permission_templatetag_test_perm1'))
        self.assertFalse(
            user.has_perm('permission.permission_templatetag_test_perm2'))

        context = Context({
            'user': user,
        })

        out = Template(
            "{% load permissionif %}"
            "{% permission user has "
            "'permission.permission_templatetag_test_perm1' "
            "and user has 'permission.permission_templatetag_test_perm2' %}"
            "Fail"
            "{% elpermission user has "
            "'permission.permission_templatetag_test_perm1' "
            "or user has 'permission.permission_templatetag_test_perm2' %}"
            "Success"
            "{% endpermission %}"
        ).render(context)

        self.assertEqual(out, "Success")


@override_settings(
    AUTHENTICATION_BACKENDS=(
        'django.contrib.auth.backends.ModelBackend',
        'permission.backends.PermissionBackend',
    ),
    PERMISSION_REPLACE_BUILTIN_IF=True,
)
class PermissionTemplateTagsWithBuiltinTestCase(TestCase):

    def setUp(self):
        # store original registry
        self._original_registry = registry._registry
        # clear registry and register mock handler
        registry._registry = {}
        # Make sure builtin if is replaced
        replace_builtin_if(True)

    def tearDown(self):
        # restore original registry
        registry._registry = self._original_registry

    def test_permissionif_tag(self):
        user = create_user('permission_templatetag_test_user1')
        perm = create_permission('permission_templatetag_test_perm1')

        user.user_permissions.add(perm)

        self.assertTrue(user.has_perm(
            'permission.permission_templatetag_test_perm1'))

        context = Context({
            'user': user,
        })

        out = Template(
            "{% permission user has "
            "'permission.permission_templatetag_test_perm1' %}"
            "Success"
            "{% else %}"
            "Fail"
            "{% endpermission %}"
        ).render(context)

        self.assertEqual(out, "Success")

    def test_permissionif_tag_elif(self):
        user = create_user('permission_templatetag_test_user1')
        perm = create_permission('permission_templatetag_test_perm1')

        user.user_permissions.add(perm)

        self.assertTrue(user.has_perm(
            'permission.permission_templatetag_test_perm1'))

        context = Context({
            'user': user,
        })

        out = Template(
            "{% permission user has 'permission.unknown_permission' %}"
            "Fail"
            "{% elpermission user has 'permission.unknown_permission2' %}"
            "Fail"
            "{% elpermission user has "
            "'permission.permission_templatetag_test_perm1' %}"
            "Success"
            "{% else %}"
            "Fail"
            "{% endpermission %}"
        ).render(context)

        self.assertEqual(out, "Success")

    def test_permissionif_tag_else(self):
        user = create_user('permission_templatetag_test_user1')
        perm = create_permission('permission_templatetag_test_perm1')

        user.user_permissions.add(perm)

        self.assertTrue(
            user.has_perm('permission.permission_templatetag_test_perm1'))

        context = Context({
            'user': user,
        })

        out = Template(
            "{% permission user has 'permission.unknown_permission' %}"
            "Fail"
            "{% else %}"
            "Success"
            "{% endpermission %}"
        ).render(context)

        self.assertEqual(out, "Success")

    def test_permissionif_tag_with_obj(self):
        from permission.tests.models import Article
        from permission.handlers import PermissionHandler

        user = create_user('permission_templatetag_test_user1')
        art1 = create_article('permission_templatetag_test_article1')
        art2 = create_article('permission_templatetag_test_article2')
        create_permission('permission_templatetag_test_perm1')

        class ArticlePermissionHandler(PermissionHandler):

            def has_perm(self, user_obj, perm, obj=None):
                if perm == 'permission.permission_templatetag_test_perm1':
                    if (obj and obj.title ==
                            'permission_templatetag_test_article2'):
                        return True
                return False
        registry.register(Article, ArticlePermissionHandler)

        self.assertFalse(
            user.has_perm('permission.permission_templatetag_test_perm1'))
        self.assertFalse(
            user.has_perm('permission.permission_templatetag_test_perm1',
                          art1))
        self.assertTrue(
            user.has_perm('permission.permission_templatetag_test_perm1',
                          art2))

        context = Context({
            'user': user,
            'art1': art1,
            'art2': art2,
        })

        out = Template(
            "{% permission user has "
            "'permission.permission_templatetag_test_perm1' %}"
            "Fail"
            "{% elpermission user has "
            "'permission.permission_templatetag_test_perm1' of art1 %}"
            "Fail"
            "{% elpermission user has "
            "'permission.permission_templatetag_test_perm1' of art2 %}"
            "Success"
            "{% else %}"
            "Fail"
            "{% endpermission %}"
        ).render(context)

        self.assertEqual(out, "Success")

    def test_permissionif_tag_and(self):
        user = create_user('permission_templatetag_test_user1')
        perm1 = create_permission('permission_templatetag_test_perm1')
        perm2 = create_permission('permission_templatetag_test_perm2')

        user.user_permissions.add(perm1, perm2)

        self.assertTrue(
            user.has_perm('permission.permission_templatetag_test_perm1'))
        self.assertTrue(
            user.has_perm('permission.permission_templatetag_test_perm2'))

        context = Context({
            'user': user,
        })

        out = Template(
            "{% permission user has 'permission.unknown_perm' "
            "and user has 'permission.permission_templatetag_test_perm2' %}"
            "Fail"
            "{% elpermission user has "
            "'permission.permission_templatetag_test_perm1' "
            "and user has 'permission.unknown_perm' %}"
            "Fail"
            "{% elpermission user has "
            "'permission.permission_templatetag_test_perm1' "
            "and user has 'permission.permission_templatetag_test_perm2' %}"
            "Success"
            "{% endpermission %}"
        ).render(context)

        self.assertEqual(out, "Success")

    def test_permissionif_tag_or(self):

        user = create_user('permission_templatetag_test_user1')
        perm1 = create_permission('permission_templatetag_test_perm1')
        create_permission('permission_templatetag_test_perm2')

        user.user_permissions.add(perm1)

        self.assertTrue(
            user.has_perm('permission.permission_templatetag_test_perm1'))
        self.assertFalse(
            user.has_perm('permission.permission_templatetag_test_perm2'))

        context = Context({
            'user': user,
        })

        out = Template(
            "{% permission user has "
            "'permission.permission_templatetag_test_perm1' "
            "and user has 'permission.permission_templatetag_test_perm2' %}"
            "Fail"
            "{% elpermission user has "
            "'permission.permission_templatetag_test_perm1' "
            "or user has 'permission.permission_templatetag_test_perm2' %}"
            "Success"
            "{% endpermission %}"
        ).render(context)

        self.assertEqual(out, "Success")

    def test_if_tag(self):
        user = create_user('permission_templatetag_test_user1')
        perm = create_permission('permission_templatetag_test_perm1')

        user.user_permissions.add(perm)

        self.assertTrue(
            user.has_perm('permission.permission_templatetag_test_perm1'))

        context = Context({
            'user': user,
        })

        out = Template(
            "{% if user has 'permission.permission_templatetag_test_perm1' %}"
            "Success"
            "{% else %}"
            "Fail"
            "{% endif %}"
        ).render(context)

        self.assertEqual(out, "Success")

    def test_if_tag_elif(self):
        user = create_user('permission_templatetag_test_user1')
        perm = create_permission('permission_templatetag_test_perm1')

        user.user_permissions.add(perm)

        self.assertTrue(
            user.has_perm('permission.permission_templatetag_test_perm1'))

        context = Context({
            'user': user,
        })

        out = Template(
            "{% if user has 'permission.unknown_permission' %}"
            "Fail"
            "{% elif user has 'permission.unknown_permisson2' %}"
            "Fail"
            "{% elif user has "
            "'permission.permission_templatetag_test_perm1' %}"
            "Success"
            "{% else %}"
            "Fail"
            "{% endif %}"
        ).render(context)

        self.assertEqual(out, "Success")

    def test_if_tag_else(self):
        user = create_user('permission_templatetag_test_user1')
        perm = create_permission('permission_templatetag_test_perm1')

        user.user_permissions.add(perm)

        self.assertTrue(
            user.has_perm('permission.permission_templatetag_test_perm1'))

        context = Context({
            'user': user,
        })

        out = Template(
            "{% if user has 'permission.unknown_permission' %}"
            "Fail"
            "{% else %}"
            "Success"
            "{% endif %}"
        ).render(context)

        self.assertEqual(out, "Success")

    def test_if_tag_with_obj(self):
        from permission.tests.models import Article
        from permission.handlers import PermissionHandler

        user = create_user('permission_templatetag_test_user1')
        art1 = create_article('permission_templatetag_test_article1')
        art2 = create_article('permission_templatetag_test_article2')
        create_permission('permission_templatetag_test_perm1')

        class ArticlePermissionHandler(PermissionHandler):
            def has_perm(self, user_obj, perm, obj=None):
                if perm == 'permission.permission_templatetag_test_perm1':
                    if (obj and obj.title ==
                            'permission_templatetag_test_article2'):
                        return True
                return False
        registry.register(Article, ArticlePermissionHandler)

        self.assertFalse(
            user.has_perm('permission.permission_templatetag_test_perm1'))
        self.assertFalse(
            user.has_perm('permission.permission_templatetag_test_perm1',
                          art1))
        self.assertTrue(
            user.has_perm('permission.permission_templatetag_test_perm1',
                          art2))

        context = Context({
            'user': user,
            'art1': art1,
            'art2': art2,
        })

        out = Template(
            "{% if user has 'permission.permission_templatetag_test_perm1' %}"
            "Fail"
            "{% elif user has "
            "'permission.permission_templatetag_test_perm1' of art1 %}"
            "Fail"
            "{% elif user has "
            "'permission.permission_templatetag_test_perm1' of art2 %}"
            "Success"
            "{% else %}"
            "Fail"
            "{% endif %}"
        ).render(context)

        self.assertEqual(out, "Success")

    def test_if_tag_and(self):
        user = create_user('permission_templatetag_test_user1')
        perm1 = create_permission('permission_templatetag_test_perm1')
        perm2 = create_permission('permission_templatetag_test_perm2')

        user.user_permissions.add(perm1, perm2)

        self.assertTrue(
            user.has_perm('permission.permission_templatetag_test_perm1'))
        self.assertTrue(
            user.has_perm('permission.permission_templatetag_test_perm2'))

        context = Context({
            'user': user,
        })

        out = Template(
            "{% if user has 'permission.unknown_perm' "
            "and user has 'permission.permission_templatetag_test_perm2' %}"
            "Fail"
            "{% elif user has 'permission.permission_templatetag_test_perm1' "
            "and user has 'permission.unknown_perm' %}"
            "Fail"
            "{% elif user has 'permission.permission_templatetag_test_perm1' "
            "and user has 'permission.permission_templatetag_test_perm2' %}"
            "Success"
            "{% endif %}"
        ).render(context)

        self.assertEqual(out, "Success")

    def test_if_tag_or(self):
        user = create_user('permission_templatetag_test_user1')
        perm1 = create_permission('permission_templatetag_test_perm1')
        create_permission('permission_templatetag_test_perm2')

        user.user_permissions.add(perm1)

        self.assertTrue(
            user.has_perm('permission.permission_templatetag_test_perm1'))
        self.assertFalse(
            user.has_perm('permission.permission_templatetag_test_perm2'))

        context = Context({
            'user': user,
        })

        out = Template(
            "{% if user has 'permission.permission_templatetag_test_perm1' "
            "and user has 'permission.permission_templatetag_test_perm2' %}"
            "Fail"
            "{% elif user has 'permission.permission_templatetag_test_perm1' "
            "or user has 'permission.permission_templatetag_test_perm2' %}"
            "Success"
            "{% endif %}"
        ).render(context)

        self.assertEqual(out, "Success")



# coding=utf-8
from django.test import TestCase, override_settings
from ..handlers import PermissionHandler
from ..handlers import LogicalPermissionHandler
from .utils import create_user, create_article
from .models import Article
from .compat import MagicMock


@override_settings(
    PERMISSION_DEFAULT_PERMISSION_HANDLER=PermissionHandler
)
class PermissionPermissionHandlersTestCase(TestCase):
    def setUp(self):
        self.handler = PermissionHandler
        self.user = create_user('john')
        self.perm1 = 'permission.add_article'
        self.perm2 = 'permission.change_article'
        self.perm3 = 'permission.delete_article'
        self.article = create_article('test')

    def test_constructor_with_model(self):
        instance = self.handler(Article)
        self.assertEqual(instance.app_label, 'permission')
        self.assertEqual(instance.model, Article)
        # backward reference
        self.assertEqual(Article._permission_handler, instance)

    def test_constructor_with_app_label(self):
        instance = self.handler('permission')
        self.assertEqual(instance.app_label, 'permission')
        self.assertEqual(instance.model, None)

    def test__get_app_perms_with_app_label(self):
        instance = self.handler('permission')
        perms = instance._get_app_perms()
        self.assertEquals(perms, set([
            'permission.add_article',
            'permission.change_article',
            'permission.delete_article',
            'permission.add_bridge',
            'permission.change_bridge',
            'permission.delete_bridge',
        ]))

    def test__get_app_perms_with_model(self):
        instance = self.handler(Article)
        perms = instance._get_app_perms()
        self.assertEquals(perms, set([
            'permission.add_article',
            'permission.change_article',
            'permission.delete_article',
            'permission.add_bridge',
            'permission.change_bridge',
            'permission.delete_bridge',
        ]))

    def test__get_model_perms(self):
        instance = self.handler(Article)
        perms = instance._get_model_perms()
        self.assertEquals(perms, set([
            'permission.add_article',
            'permission.change_article',
            'permission.delete_article',
        ]))

    def test_get_supported_permissions(self):
        instance = self.handler(Article)
        perms = instance.get_supported_permissions()
        self.assertEquals(perms, set([
            'permission.add_article',
            'permission.change_article',
            'permission.delete_article',
        ]))

    def test_get_supported_permissions_with_includes(self):
        instance = self.handler(Article)
        instance.includes = [
            'permission.add_article',
            'permission.change_article',
        ]
        perms = instance.get_supported_permissions()
        self.assertEquals(perms, set([
            'permission.add_article',
            'permission.change_article',
        ]))

    def test_get_supported_permissions_with_includes_change(self):
        instance = self.handler(Article)
        instance.includes = [
            'permission.add_article',
            'permission.change_article',
        ]
        instance.get_supported_permissions()
        instance.includes = [
            'permission.change_article',
        ]
        perms = instance.get_supported_permissions()
        self.assertEquals(perms, set([
            'permission.change_article',
        ]))

    def test_get_supported_permissions_with_excludes(self):
        instance = self.handler(Article)
        instance.excludes = [
            'permission.add_article',
        ]
        perms = instance.get_supported_permissions()
        self.assertEquals(perms, set([
            'permission.change_article',
            'permission.delete_article',
        ]))

    def test_get_supported_permissions_with_excludes_change(self):
        instance = self.handler(Article)
        instance.excludes = [
            'permission.add_article',
        ]
        instance.get_supported_permissions()
        instance.excludes = []
        perms = instance.get_supported_permissions()
        self.assertEquals(perms, set([
            'permission.add_article',
            'permission.change_article',
            'permission.delete_article',
        ]))

    def test_get_supported_app_labels(self):
        instance = self.handler(Article)
        app_labels = instance.get_supported_app_labels()
        self.assertEquals(app_labels, set([
            'permission',
        ]))

    def test_get_supported_app_labels_with_includes(self):
        instance = self.handler(Article)
        instance.includes = [
            'permission.add_article',
            'permission.change_article',
        ]
        app_labels = instance.get_supported_app_labels()
        self.assertEquals(app_labels, set([
            'permission',
        ]))

    def test_get_supported_app_labels_with_includes_change(self):
        instance = self.handler(Article)
        instance.includes = [
            'permission.add_article',
            'permission.change_article',
        ]
        instance.get_supported_app_labels()
        instance.includes = [
            'permission.change_article',
        ]
        app_labels = instance.get_supported_app_labels()
        self.assertEquals(app_labels, set([
            'permission',
        ]))

    def test_get_supported_app_labels_with_excludes(self):
        instance = self.handler(Article)
        instance.excludes = [
            'permission.add_article',
        ]
        app_labels = instance.get_supported_app_labels()
        self.assertEquals(app_labels, set([
            'permission',
        ]))

    def test_get_supported_app_labels_with_excludes_change(self):
        instance = self.handler(Article)
        instance.excludes = [
            'permission.add_article',
        ]
        instance.get_supported_app_labels()
        instance.excludes = []
        app_labels = instance.get_supported_app_labels()
        self.assertEquals(app_labels, set([
            'permission',
        ]))

    def test_has_perm_add_wihtout_obj(self):
        instance = self.handler(Article)
        self.assertRaises(
            NotImplementedError,
            instance.has_perm,
            self.user, self.perm1)

    def test_has_perm_change_wihtout_obj(self):
        instance = self.handler(Article)
        self.assertRaises(
            NotImplementedError,
            instance.has_perm,
            self.user, self.perm2)

    def test_has_perm_delete_wihtout_obj(self):
        instance = self.handler(Article)
        self.assertRaises(
            NotImplementedError,
            instance.has_perm,
            self.user, self.perm3)

    def test_has_perm_add_wiht_obj(self):
        instance = self.handler(Article)
        self.assertRaises(
            NotImplementedError,
            instance.has_perm,
            self.user, self.perm1, self.article)

    def test_has_perm_change_wiht_obj(self):
        instance = self.handler(Article)
        self.assertRaises(
            NotImplementedError,
            instance.has_perm,
            self.user, self.perm2, self.article)

    def test_has_perm_delete_wiht_obj(self):
        instance = self.handler(Article)
        self.assertRaises(
            NotImplementedError,
            instance.has_perm,
            self.user, self.perm3, self.article)

    def test_has_module_perms_success(self):
        instance = self.handler(Article)
        user = MagicMock()
        user.has_perm.return_value = True
        self.assertTrue(instance.has_module_perms(user, 'permission'))
        self.assertTrue(user.has_perm.called)

    def test_has_module_perms_fail(self):
        instance = self.handler(Article)
        user = MagicMock()
        user.has_perm.return_value = True
        self.assertFalse(instance.has_module_perms(user, 'unknown'))
        self.assertFalse(user.has_perm.called)


@override_settings(
    PERMISSION_DEFAULT_PERMISSION_HANDLER=LogicalPermissionHandler,
)
class PermissionLogicalPermissionHandlerTestCase(TestCase):
    def setUp(self):
        # make sure all caches are removed
        Article._permission_logics = set()
        self.handler = LogicalPermissionHandler
        self.user = create_user('john')
        self.perm1 = 'permission.add_article'
        self.perm2 = 'permission.change_article'
        self.perm3 = 'permission.delete_article'
        self.article = create_article('test')

        from permission.logics import PermissionLogic
        from permission import add_permission_logic
        self.mock_logic1 = MagicMock(spec=PermissionLogic)
        self.mock_logic1.has_perm = MagicMock(return_value=False)
        self.mock_logic2 = MagicMock(spec=PermissionLogic)
        self.mock_logic2.has_perm = MagicMock(return_value=False)
        add_permission_logic(Article, self.mock_logic1)
        add_permission_logic(Article, self.mock_logic2)

    def test_constructor_with_app_label(self):
        self.assertRaises(AttributeError,
                          self.handler, 'permission')

    def test_has_perm_non_related_permission(self):
        instance = self.handler(Article)
        instance.get_supported_permissions = MagicMock(return_value=[
            'permission.add_article',
            'permission.change_article',
            'permission.delete_article',
        ])
        self.assertFalse(instance.has_perm(self.user, 'unknown'))
        self.assertFalse(instance.has_perm(self.user, 'unknown', self.article))

    def test_has_perm_permission_logics_called(self):
        instance = self.handler(Article)
        instance.get_supported_permissions = MagicMock(return_value=[
            'permission.add_article',
            'permission.change_article',
            'permission.delete_article',
        ])
        self.assertFalse(self.mock_logic1.has_perm.called)
        self.assertFalse(self.mock_logic2.has_perm.called)
        self.assertFalse(instance.has_perm(self.user,
                                           'permission.add_article'))
        self.assertTrue(self.mock_logic1.has_perm.called)
        self.assertTrue(self.mock_logic2.has_perm.called)
        self.assertEqual(self.mock_logic1.has_perm.call_count, 1)
        self.assertEqual(self.mock_logic2.has_perm.call_count, 1)
        # permission check should be cached thus `has_perm` should not be
        # called twice for same user instance
        self.assertFalse(instance.has_perm(self.user,
                                           'permission.add_article'))
        self.assertEqual(self.mock_logic1.has_perm.call_count, 1)
        self.assertEqual(self.mock_logic2.has_perm.call_count, 1)



from django.test import TestCase, override_settings
from django.core.exceptions import ObjectDoesNotExist
from ..backends import PermissionBackend
from ..utils.handlers import registry
from .compat import MagicMock
from .utils import create_user, create_article
from .models import Article


@override_settings(
    AUTHENTICATION_BACKENDS=(
        'django.contrib.auth.backends.ModelBackend',
        'permission.backends.PermissionBackend',
    ),
)
class PermissionPermissionBackendTestCase(TestCase):
    def setUp(self):
        self.user = create_user('john')
        self.perm1 = 'permission.add_article'
        self.perm2 = 'permission.change_article'
        self.perm3 = 'permission.delete_article'
        self.article = create_article('test')
        self.original_get_handlers = registry.get_handlers

    def tearDown(self):
        registry.get_handlers = self.original_get_handlers

    def test_constructor(self):
        backend = PermissionBackend()

    def test_authenticate(self):
        backend = PermissionBackend()
        self.assertEqual(backend.authenticate(None, None), None)

    def test_has_perm_without_obj(self):
        perms = [
            'permission.add_article',
            'permission.change_article',
            'permission.delete_article',
        ]
        registry.get_handlers = MagicMock(return_value=[
            MagicMock(get_supported_permissions=MagicMock(return_value=perms),
                      has_perm=MagicMock(return_value=False)),
            MagicMock(get_supported_permissions=MagicMock(return_value=perms),
                      has_perm=MagicMock(return_value=True)),
        ])

        backend = PermissionBackend()
        self.assertFalse(registry.get_handlers.called)
        self.assertFalse(registry.get_handlers()[0].get_supported_permissions.called)
        self.assertFalse(registry.get_handlers()[1].get_supported_permissions.called)
        self.assertFalse(registry.get_handlers()[0].has_perm.called)
        self.assertFalse(registry.get_handlers()[1].has_perm.called)

        self.assertTrue(backend.has_perm(self.user, self.perm1))

        self.assertTrue(registry.get_handlers.called)
        self.assertTrue(registry.get_handlers()[0].get_supported_permissions.called)
        self.assertTrue(registry.get_handlers()[1].get_supported_permissions.called)
        self.assertTrue(registry.get_handlers()[0].has_perm.called)
        self.assertTrue(registry.get_handlers()[1].has_perm.called)

    def test_has_perm_with_obj(self):
        perms = [
            'permission.add_article',
            'permission.change_article',
            'permission.delete_article',
        ]
        registry.get_handlers = MagicMock(return_value=[
            MagicMock(get_supported_permissions=MagicMock(return_value=perms),
                      has_perm=MagicMock(return_value=False)),
            MagicMock(get_supported_permissions=MagicMock(return_value=perms),
                      has_perm=MagicMock(return_value=True)),
        ])

        backend = PermissionBackend()
        self.assertFalse(registry.get_handlers.called)
        self.assertFalse(registry.get_handlers()[0].get_supported_permissions.called)
        self.assertFalse(registry.get_handlers()[1].get_supported_permissions.called)
        self.assertFalse(registry.get_handlers()[0].has_perm.called)
        self.assertFalse(registry.get_handlers()[1].has_perm.called)

        self.assertTrue(backend.has_perm(self.user, self.perm1, self.article))

        self.assertTrue(registry.get_handlers.called)
        self.assertTrue(registry.get_handlers()[0].get_supported_permissions.called)
        self.assertTrue(registry.get_handlers()[1].get_supported_permissions.called)
        self.assertTrue(registry.get_handlers()[0].has_perm.called)
        self.assertTrue(registry.get_handlers()[1].has_perm.called)

    @override_settings(
        PERMISSION_CHECK_PERMISSION_PRESENCE=False,
    )
    def test_has_perm_with_nil_permission(self):
        perms = [
            'permission.add_article',
            'permission.change_article',
            'permission.delete_article',
        ]
        registry.get_handlers = MagicMock(return_value=[
            MagicMock(get_permissions=MagicMock(return_value=perms),
                      has_perm=MagicMock(return_value=False)),
            MagicMock(get_permissions=MagicMock(return_value=perms),
                      has_perm=MagicMock(return_value=True)),
        ])

        backend = PermissionBackend()
        self.assertFalse(backend.has_perm(None, 'permissions.nil_permission'))

    @override_settings(
        PERMISSION_CHECK_PERMISSION_PRESENCE=True,
    )
    def test_has_perm_with_nil_permission_raise(self):
        perms = [
            'permission.add_article',
            'permission.change_article',
            'permission.delete_article',
        ]
        registry.get_handlers = MagicMock(return_value=[
            MagicMock(get_permissions=MagicMock(return_value=perms),
                      has_perm=MagicMock(return_value=False)),
            MagicMock(get_permissions=MagicMock(return_value=perms),
                      has_perm=MagicMock(return_value=True)),
        ])

        backend = PermissionBackend()
        self.assertRaises(ObjectDoesNotExist,
                backend.has_perm,
                None, 'permissions.nil_permission')

    @override_settings(
        PERMISSION_CHECK_PERMISSION_PRESENCE=False,
        AUTHENTICATION_BACKENDS = (
            'django.contrib.auth.backends.ModelBackend',
            'permission.backends.PermissionBackend',
        ),
    )
    def test_has_perm_with_nil_permission_with_user(self):
        self.assertFalse(self.user.has_perm('permissions.nil_permission'))

    @override_settings(
        PERMISSION_CHECK_PERMISSION_PRESENCE=True,
        AUTHENTICATION_BACKENDS = (
            'django.contrib.auth.backends.ModelBackend',
            'permission.backends.PermissionBackend',
        ),
    )
    def test_has_perm_with_nil_permission_raise_with_user(self):
        self.assertRaises(ObjectDoesNotExist,
                self.user.has_perm,
                'permissions.nil_permission')

    def test_has_module_perms(self):
        perms = [
            'permission.add_article',
            'permission.change_article',
            'permission.delete_article',
        ]
        app_labels = ['permission']
        registry.get_handlers = MagicMock(return_value=[
            MagicMock(get_supported_app_labels=MagicMock(return_value=app_labels),
                      has_module_perms=MagicMock(return_value=False)),
            MagicMock(get_supported_app_labels=MagicMock(return_value=app_labels),
                      has_module_perms=MagicMock(return_value=True)),
        ])

        backend = PermissionBackend()
        self.assertFalse(registry.get_handlers.called)
        self.assertFalse(registry.get_handlers()[0].get_supported_app_labels.called)
        self.assertFalse(registry.get_handlers()[1].get_supported_app_labels.called)
        self.assertFalse(registry.get_handlers()[0].has_module_perms.called)
        self.assertFalse(registry.get_handlers()[1].has_module_perms.called)

        self.assertTrue(backend.has_module_perms(self.user, 'permission'))

        self.assertTrue(registry.get_handlers.called)
        self.assertTrue(registry.get_handlers()[0].get_supported_app_labels.called)
        self.assertTrue(registry.get_handlers()[1].get_supported_app_labels.called)
        self.assertTrue(registry.get_handlers()[0].has_module_perms.called)
        self.assertTrue(registry.get_handlers()[1].has_module_perms.called)



# coding=utf-8
from django.test import TestCase
from django.core.exceptions import PermissionDenied
from ...utils.handlers import registry
from ...decorators.methodbase import permission_required
from ..compat import MagicMock
from .utils import (
    create_mock_handler,
    create_mock_request,
    create_mock_queryset,
    create_mock_model,
    create_mock_view_func,
    create_mock_view_class,
)


class PermissionClassDecoratorsTestCase(TestCase):
    def setUp(self):
        self.handler = create_mock_handler()
        self.request = create_mock_request(self.handler)
        self.model = create_mock_model()
        self.model_instance = self.model()
        self.queryset = create_mock_queryset(self.model_instance)

        self.view_func = create_mock_view_func()
        self.view_class = create_mock_view_class(
            permission_required(
                'permission.add_article')(self.view_func))
        self.view_class_exc = create_mock_view_class(
            permission_required(
                'permission.add_article',
                raise_exception=True,
            )(self.view_func))

        # store original registry
        self._original_registry = registry._registry

        # clear registry and register mock handler
        registry._registry = {}
        registry.register(
                self.model,
                self.handler,
            )

        # clear call history
        self.handler.has_perm.return_value = False

    def tearDown(self):
        # restore original registry
        registry._registry = self._original_registry

    def test_with_object(self):
        # set object
        self.view_class.object = self.model_instance
        self.view_class_exc.object = self.model_instance

        # has_perm always return False
        self.view_class.as_view()(self.request, pk=1)
        self.request.user.has_perm.assert_called_with(
                'permission.add_article',
                obj=self.model_instance,
            )
        self.handler.has_perm.assert_called_with(
                'permission.add_article',
                obj=self.model_instance,
            )
        self.assertFalse(self.view_func.called)

        self.assertRaises(PermissionDenied,
                          self.view_class_exc.as_view(),
                          self.request,
                          pk=1)
        self.assertFalse(self.view_func.called)

        # has_perm always return True
        self.handler.has_perm.return_value = True
        self.view_class.as_view()(self.request, pk=1)
        self.request.user.has_perm.assert_called_with(
                'permission.add_article',
                obj=self.model_instance,
            )
        self.handler.has_perm.assert_called_with(
                'permission.add_article',
                obj=self.model_instance,
            )
        self.assertTrue(self.view_func.called)

    def test_with_get_object(self):
        # set object
        self.view_class.get_object = MagicMock(
                return_value=self.model_instance)
        self.view_class_exc.get_object = MagicMock(
                return_value=self.model_instance)

        # has_perm always return False
        self.view_class.as_view()(self.request, pk=1)
        self.request.user.has_perm.assert_called_with(
                'permission.add_article',
                obj=self.model_instance,
            )
        self.handler.has_perm.assert_called_with(
                'permission.add_article',
                obj=self.model_instance,
            )
        self.assertFalse(self.view_func.called)

        self.assertRaises(PermissionDenied,
                          self.view_class_exc.as_view(),
                          self.request,
                          pk=1)
        self.assertFalse(self.view_func.called)

        # has_perm always return True
        self.handler.has_perm.return_value = True
        self.view_class.as_view()(self.request, pk=1)
        self.request.user.has_perm.assert_called_with(
                'permission.add_article',
                obj=self.model_instance,
            )
        self.handler.has_perm.assert_called_with(
                'permission.add_article',
                obj=self.model_instance,
            )
        self.assertTrue(self.view_func.called)

    def test_with_queryset(self):
        # set object
        get_object = lambda x, y: y.get(*x.args, **x.kwargs)
        self.view_class.get_object = get_object
        self.view_class_exc.get_object = get_object
        self.view_class.queryset = self.queryset
        self.view_class_exc.queryset = self.queryset

        # has_perm always return False
        self.view_class.as_view()(self.request, pk=1)
        self.request.user.has_perm.assert_called_with(
                'permission.add_article',
                obj=self.model_instance,
            )
        self.handler.has_perm.assert_called_with(
                'permission.add_article',
                obj=self.model_instance,
            )
        self.assertFalse(self.view_func.called)

        self.assertRaises(PermissionDenied,
                          self.view_class_exc.as_view(),
                          self.request,
                          pk=1)
        self.assertFalse(self.view_func.called)

        # has_perm always return True
        self.handler.has_perm.return_value = True
        self.view_class.as_view()(self.request, pk=1)
        self.request.user.has_perm.assert_called_with(
                'permission.add_article',
                obj=self.model_instance,
            )
        self.handler.has_perm.assert_called_with(
                'permission.add_article',
                obj=self.model_instance,
            )
        self.assertTrue(self.view_func.called)

    def test_with_get_queryset(self):
        # set object
        get_object = lambda x, y: y.get(*x.args, **x.kwargs)
        self.view_class.get_object = get_object
        self.view_class_exc.get_object = get_object
        self.view_class.get_queryset = MagicMock(
                return_value=self.queryset)
        self.view_class_exc.get_queryset = MagicMock(
                return_value=self.queryset)

        # has_perm always return False
        self.view_class.as_view()(self.request, pk=1)
        self.request.user.has_perm.assert_called_with(
                'permission.add_article',
                obj=self.model_instance,
            )
        self.handler.has_perm.assert_called_with(
                'permission.add_article',
                obj=self.model_instance,
            )
        self.assertFalse(self.view_func.called)

        self.assertRaises(PermissionDenied,
                          self.view_class_exc.as_view(),
                          self.request,
                          pk=1)
        self.assertFalse(self.view_func.called)

        # has_perm always return True
        self.handler.has_perm.return_value = True
        self.view_class.as_view()(self.request, pk=1)
        self.request.user.has_perm.assert_called_with(
                'permission.add_article',
                obj=self.model_instance,
            )
        self.handler.has_perm.assert_called_with(
                'permission.add_article',
                obj=self.model_instance,
            )
        self.assertTrue(self.view_func.called)




"""A module to lookup field of object."""
from __future__ import unicode_literals
from collections import Iterable


def field_lookup(obj, field_path):
    """
    Lookup django model field in similar way of django query lookup.

    Args:
        obj (instance): Django Model instance
        field_path (str): '__' separated field path

    Example:
        >>> from django.db import model
        >>> from django.contrib.auth.models import User
        >>> class Article(models.Model):
        >>>     title = models.CharField('title', max_length=200)
        >>>     author = models.ForeignKey(User, null=True,
        >>>             related_name='permission_test_articles_author')
        >>>     editors = models.ManyToManyField(User,
        >>>             related_name='permission_test_articles_editors')
        >>> user = User.objects.create_user('test_user', 'password')
        >>> article = Article.objects.create(title='test_article',
        ...                                  author=user)
        >>> article.editors.add(user)
        >>> assert 'test_article' == field_lookup(article, 'title')
        >>> assert 'test_user' == field_lookup(article, 'user__username')
        >>> assert ['test_user'] == list(field_lookup(article,
        ...                                           'editors__username'))
    """
    if hasattr(obj, 'iterator'):
        return (field_lookup(x, field_path) for x in obj.iterator())
    elif isinstance(obj, Iterable):
        return (field_lookup(x, field_path) for x in iter(obj))
    # split the path
    field_path = field_path.split('__', 1)
    if len(field_path) == 1:
        return getattr(obj, field_path[0], None)
    return field_lookup(field_lookup(obj, field_path[0]), field_path[1])



# coding=utf-8
"""
Permission utility module.

In this module, term *perm* indicate the identifier string permission written
in 'app_label.codename' format.
"""
from __future__ import unicode_literals
from django.utils.six import string_types


def get_perm_codename(perm, fail_silently=True):
    """
    Get permission codename from permission-string.

    Examples
    --------
    >>> get_perm_codename('app_label.codename_model')
    'codename_model'
    >>> get_perm_codename('app_label.codename')
    'codename'
    >>> get_perm_codename('codename_model')
    'codename_model'
    >>> get_perm_codename('codename')
    'codename'
    >>> get_perm_codename('app_label.app_label.codename_model')
    'app_label.codename_model'
    """
    try:
        perm = perm.split('.', 1)[1]
    except IndexError as e:
        if not fail_silently:
            raise e
    return perm


def permission_to_perm(permission):
    """
    Convert a permission instance to a permission-string.

    Examples
    --------
    >>> permission = Permission.objects.get(
    ...     content_type__app_label='auth',
    ...     codename='add_user',
    ... )
    >>> permission_to_perm(permission)
    'auth.add_user'
    """
    app_label = permission.content_type.app_label
    codename = permission.codename
    return '%s.%s' % (app_label, codename)


def perm_to_permission(perm):
    """
    Convert a permission-string to a permission instance.

    Examples
    --------
    >>> permission = perm_to_permission('auth.add_user')
    >>> permission.content_type.app_label
    'auth'
    >>> permission.codename
    'add_user'
    """
    from django.contrib.auth.models import Permission
    try:
        app_label, codename = perm.split('.', 1)
    except (ValueError, IndexError):
        raise AttributeError(
            'The format of identifier string permission (perm) is wrong. '
            "It should be in 'app_label.codename'."
        )
    else:
        permission = Permission.objects.get(
            content_type__app_label=app_label,
            codename=codename
        )
        return permission


def get_app_perms(model_or_app_label):
    """
    Get permission-string list of the specified django application.

    Parameters
    ----------
    model_or_app_label : model class or string
        A model class or app_label string to specify the particular django
        application.

    Returns
    -------
    set
        A set of perms of the specified django application.

    Examples
    --------
    >>> perms1 = get_app_perms('auth')
    >>> perms2 = get_app_perms(Permission)
    >>> perms1 == perms2
    True
    """
    from django.contrib.auth.models import Permission
    if isinstance(model_or_app_label, string_types):
        app_label = model_or_app_label
    else:
        # assume model_or_app_label is model class
        app_label = model_or_app_label._meta.app_label
    qs = Permission.objects.filter(content_type__app_label=app_label)
    perms = ('%s.%s' % (app_label, p.codename) for p in qs.iterator())
    return set(perms)


def get_model_perms(model):
    """
    Get permission-string list of a specified django model.

    Parameters
    ----------
    model : model class
        A model class to specify the particular django model.

    Returns
    -------
    set
        A set of perms of the specified django model.

    Examples
    --------
    >>> sorted(get_model_perms(Permission)) == [
    ...     'auth.add_permission',
    ...     'auth.change_permission',
    ...     'auth.delete_permission'
    ... ]
    True
    """
    from django.contrib.auth.models import Permission
    app_label = model._meta.app_label
    model_name = model._meta.object_name.lower()
    qs = Permission.objects.filter(content_type__app_label=app_label,
                                   content_type__model=model_name)
    perms = ('%s.%s' % (app_label, p.codename) for p in qs.iterator())
    return set(perms)



# coding=utf-8
"""
permission_required decorator for generic classbased view from django 1.3
"""
from functools import wraps
from django.utils.decorators import available_attrs
from django.core.exceptions import PermissionDenied
from permission.decorators.utils import redirect_to_login


def permission_required(perm, queryset=None,
                        login_url=None, raise_exception=False):
    """
    Permission check decorator for classbased generic view

    This decorator works as class decorator
    DO NOT use ``method_decorator`` or whatever while this decorator will use
    ``self`` argument for method of classbased generic view.

    Parameters
    ----------
    perm : string
        A permission string
    queryset : queryset or model
        A queryset or model for finding object.
        With classbased generic view, ``None`` for using view default queryset.
        When the view does not define ``get_queryset``, ``queryset``,
        ``get_object``, or ``object`` then ``obj=None`` is used to check
        permission.
        With functional generic view, ``None`` for using passed queryset.
        When non queryset was passed then ``obj=None`` is used to check
        permission.

    Examples
    --------
    >>> @permission_required('auth.change_user')
    >>> class UpdateAuthUserView(UpdateView):
    ...     pass
    """
    def wrapper(cls):
        def view_wrapper(view_func):
            @wraps(view_func, assigned=available_attrs(view_func))
            def inner(self, request, *args, **kwargs):
                # get object
                obj = get_object_from_classbased_instance(
                        self, queryset, request, *args, **kwargs
                    )

                if not request.user.has_perm(perm, obj=obj):
                    if raise_exception:
                        raise PermissionDenied
                    else:
                        return redirect_to_login(request, login_url)
                return view_func(self, request, *args, **kwargs)
            return inner
        cls.dispatch = view_wrapper(cls.dispatch)
        return cls
    return wrapper


def get_object_from_classbased_instance(
        instance, queryset, request, *args, **kwargs):
    """
    Get object from an instance of classbased generic view

    Parameters
    ----------
    instance : instance
        An instance of classbased generic view
    queryset : instance
        A queryset instance
    request : instance
        A instance of HttpRequest

    Returns
    -------
    instance
        An instance of model object or None
    """
    from django.views.generic.edit import BaseCreateView
    # initialize request, args, kwargs of classbased_instance
    # most of methods of classbased view assumed these attributes
    # but these attributes is initialized in ``dispatch`` method.
    instance.request = request
    instance.args = args
    instance.kwargs = kwargs

    # get queryset from class if ``queryset_or_model`` is not specified
    if hasattr(instance, 'get_queryset') and not queryset:
        queryset = instance.get_queryset()
    elif hasattr(instance, 'queryset') and not queryset:
        queryset = instance.queryset
    elif hasattr(instance, 'model') and not queryset:
        queryset = instance.model._default_manager.all()

    # get object
    if hasattr(instance, 'get_object'):
        try:
            obj = instance.get_object(queryset)
        except AttributeError as e:
            # CreateView has ``get_object`` method but CreateView
            # should not have any object before thus simply set
            # None
            if isinstance(instance, BaseCreateView):
                obj = None
            else:
                raise e
    elif hasattr(instance, 'object'):
        obj = instance.object
    else:
        obj = None
    return obj




# coding=utf-8
"""
permission_required decorator for generic classbased/functionbased view
"""
from functools import wraps
from django.http import HttpRequest
from django.utils.decorators import available_attrs
from django.core.exceptions import PermissionDenied
from permission.decorators.utils import redirect_to_login


def permission_required(perm, queryset=None,
                        login_url=None, raise_exception=False):
    """
    Permission check decorator for classbased/functionbased generic view

    This decorator works as method or function decorator
    DO NOT use ``method_decorator`` or whatever while this decorator will use
    ``self`` argument for method of classbased generic view.

    Parameters
    ----------
    perm : string
        A permission string
    queryset : queryset or model
        A queryset or model for finding object.
        With classbased generic view, ``None`` for using view default queryset.
        When the view does not define ``get_queryset``, ``queryset``,
        ``get_object``, or ``object`` then ``obj=None`` is used to check
        permission.
        With functional generic view, ``None`` for using passed queryset.
        When non queryset was passed then ``obj=None`` is used to check
        permission.

    Examples
    --------
    >>> # As method decorator
    >>> class UpdateAuthUserView(UpdateView):
    >>>     @permission_required('auth.change_user')
    >>>     def dispatch(self, request, *args, **kwargs):
    ...         pass
    >>> # As function decorator
    >>> @permission_required('auth.change_user')
    >>> def update_auth_user(request, *args, **kwargs):
    ...     pass
    """
    def wrapper(view_method):
        @wraps(view_method, assigned=available_attrs(view_method))
        def inner(self, request=None, *args, **kwargs):
            if isinstance(self, HttpRequest):
                from permission.decorators.functionbase import \
                        permission_required as decorator
                # this is a functional view not classbased view.
                decorator = decorator(perm, queryset,
                                      login_url, raise_exception)
                decorator = decorator(view_method)
                if not request:
                    args = list(args)
                    args.insert(0, request)
                request = self
                return decorator(request, *args, **kwargs)
            else:
                from permission.decorators.classbase import \
                        get_object_from_classbased_instance
                # get object
                obj = get_object_from_classbased_instance(
                        self, queryset, request, *args, **kwargs
                    )

                if not request.user.has_perm(perm, obj=obj):
                    if raise_exception:
                        raise PermissionDenied
                    else:
                        return redirect_to_login(request, login_url)
                return view_method(self, request, *args, **kwargs)
        return inner
    return wrapper



from django.test import TestCase, override_settings
from ... import add_permission_logic
from ...logics import CollaboratorsPermissionLogic
from ..utils import create_user, create_anonymous, create_article
from ..compat import MagicMock


@override_settings(
    PERMISSION_DEFAULT_CPL_FIELD_NAME='authors',
    PERMISSION_DEFAULT_CPL_ANY_PERMISSION=True,
    PERMISSION_DEFAULT_CPL_CHANGE_PERMISSION=True,
    PERMISSION_DEFAULT_CPL_DELETE_PERMISSION=True,
)
class PermissionLogicsCollaboratorsPermissionLogicTestCase(TestCase):
    def setUp(self):
        self.user1 = create_user('john')
        self.user2 = create_user('tony')
        self.anonymous = create_anonymous()
        self.perm1 = 'permission.add_article'
        self.perm2 = 'permission.change_article'
        self.perm3 = 'permission.delete_article'
        self.article = create_article('test')
        self.article.authors.add(self.user2)

    def test_constructor(self):
        permission_logic = CollaboratorsPermissionLogic()
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertTrue(isinstance(permission_logic, CollaboratorsPermissionLogic))
        self.assertEqual(permission_logic.field_name, 'authors')
        self.assertEqual(permission_logic.any_permission, True)
        self.assertEqual(permission_logic.change_permission, True)
        self.assertEqual(permission_logic.delete_permission, True)

    def test_constructor_with_specifing_field_name(self):
        permission_logic = CollaboratorsPermissionLogic(field_name='specified')
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertTrue(isinstance(permission_logic, CollaboratorsPermissionLogic))
        self.assertEqual(permission_logic.field_name, 'specified')

    def test_constructor_with_specifing_any_permission(self):
        permission_logic = CollaboratorsPermissionLogic(any_permission=False)
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertTrue(isinstance(permission_logic, CollaboratorsPermissionLogic))
        self.assertEqual(permission_logic.any_permission, False)

    def test_constructor_with_specifing_change_permission(self):
        permission_logic = CollaboratorsPermissionLogic(change_permission=False)
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertTrue(isinstance(permission_logic, CollaboratorsPermissionLogic))
        self.assertEqual(permission_logic.change_permission, False)

    def test_constructor_with_specifing_delete_permission(self):
        permission_logic = CollaboratorsPermissionLogic(delete_permission=False)
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertTrue(isinstance(permission_logic, CollaboratorsPermissionLogic))
        self.assertEqual(permission_logic.delete_permission, False)

    def test_has_perm_add_without_obj(self):
        permission_logic = CollaboratorsPermissionLogic()
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertTrue(permission_logic.has_perm(self.user1, self.perm1))

    def test_has_perm_change_without_obj(self):
        permission_logic = CollaboratorsPermissionLogic()
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertTrue(permission_logic.has_perm(self.user1, self.perm2))

    def test_has_perm_delete_without_obj(self):
        permission_logic = CollaboratorsPermissionLogic()
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertTrue(permission_logic.has_perm(self.user1, self.perm3))

    def test_has_perm_add_with_obj(self):
        permission_logic = CollaboratorsPermissionLogic()
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertFalse(
                permission_logic.has_perm(self.user1, self.perm1, self.article))

    def test_has_perm_change_with_obj(self):
        permission_logic = CollaboratorsPermissionLogic()
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertFalse(
                permission_logic.has_perm(self.user1, self.perm2, self.article))

    def test_has_perm_delete_with_obj(self):
        permission_logic = CollaboratorsPermissionLogic()
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertFalse(
                permission_logic.has_perm(self.user1, self.perm3, self.article))

    def test_has_perm_add_without_obj_with_anonymous(self):
        permission_logic = CollaboratorsPermissionLogic()
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertFalse(permission_logic.has_perm(self.anonymous, self.perm1))

    def test_has_perm_change_without_obj_with_anonymous(self):
        permission_logic = CollaboratorsPermissionLogic()
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertFalse(permission_logic.has_perm(self.anonymous, self.perm2))

    def test_has_perm_delete_without_obj_with_anonymous(self):
        permission_logic = CollaboratorsPermissionLogic()
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertFalse(permission_logic.has_perm(self.anonymous, self.perm3))

    def test_has_perm_add_with_obj_with_anonymous(self):
        permission_logic = CollaboratorsPermissionLogic()
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertFalse(
            permission_logic.has_perm(self.anonymous, self.perm1, self.article))

    def test_has_perm_change_with_obj_with_anonymous(self):
        permission_logic = CollaboratorsPermissionLogic()
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertFalse(
            permission_logic.has_perm(self.anonymous, self.perm2, self.article))

    def test_has_perm_delete_with_obj_with_anonymous(self):
        permission_logic = CollaboratorsPermissionLogic()
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertFalse(
            permission_logic.has_perm(self.anonymous, self.perm3, self.article))

    def test_has_perm_add_with_obj_collaborators(self):
        permission_logic = CollaboratorsPermissionLogic()
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertTrue(
                permission_logic.has_perm(self.user2, self.perm1, self.article))

    def test_has_perm_change_with_obj_collaborators(self):
        permission_logic = CollaboratorsPermissionLogic()
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertTrue(
                permission_logic.has_perm(self.user2, self.perm2, self.article))

    def test_has_perm_delete_with_obj_collaborators(self):
        permission_logic = CollaboratorsPermissionLogic()
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertTrue(
                permission_logic.has_perm(self.user2, self.perm3, self.article))

    def test_has_perm_add_with_obj_collaborators_diff_field_name(self):
        permission_logic = CollaboratorsPermissionLogic(field_name='editors')
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertFalse(
                permission_logic.has_perm(self.user2, self.perm1, self.article))

    def test_has_perm_change_with_obj_collaborators_diff_field_name(self):
        permission_logic = CollaboratorsPermissionLogic(field_name='editors')
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertFalse(
                permission_logic.has_perm(self.user2, self.perm2, self.article))

    def test_has_perm_delete_with_obj_collaborators_diff_field_name(self):
        permission_logic = CollaboratorsPermissionLogic(field_name='editors')
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertFalse(
                permission_logic.has_perm(self.user2, self.perm3, self.article))

    def test_has_perm_add_with_obj_collaborators_non_any(self):
        permission_logic = CollaboratorsPermissionLogic(any_permission=False)
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertFalse(
                permission_logic.has_perm(self.user2, self.perm1, self.article))

    def test_has_perm_change_with_obj_collaborators_non_any(self):
        permission_logic = CollaboratorsPermissionLogic(any_permission=False)
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertTrue(
                permission_logic.has_perm(self.user2, self.perm2, self.article))

    def test_has_perm_delete_with_obj_non_any(self):
        permission_logic = CollaboratorsPermissionLogic(any_permission=False)
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertTrue(
                permission_logic.has_perm(self.user2, self.perm3, self.article))

    def test_has_perm_add_with_obj_collaborators_non_any_no_change(self):
        permission_logic = CollaboratorsPermissionLogic(any_permission=False,
                                                 change_permission=False)
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertFalse(
                permission_logic.has_perm(self.user2, self.perm1, self.article))

    def test_has_perm_change_with_obj_collaborators_non_any_no_change(self):
        permission_logic = CollaboratorsPermissionLogic(any_permission=False,
                                                 change_permission=False)
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertFalse(
                permission_logic.has_perm(self.user2, self.perm2, self.article))

    def test_has_perm_delete_with_obj_non_any_no_change(self):
        permission_logic = CollaboratorsPermissionLogic(any_permission=False,
                                                 change_permission=False)
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertTrue(
                permission_logic.has_perm(self.user2, self.perm3, self.article))

    def test_has_perm_add_with_obj_collaborators_non_any_no_delete(self):
        permission_logic = CollaboratorsPermissionLogic(any_permission=False,
                                                 delete_permission=False)
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertFalse(
                permission_logic.has_perm(self.user2, self.perm1, self.article))

    def test_has_perm_change_with_obj_collaborators_non_any_no_delete(self):
        permission_logic = CollaboratorsPermissionLogic(any_permission=False,
                                                 delete_permission=False)
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertTrue(
                permission_logic.has_perm(self.user2, self.perm2, self.article))

    def test_has_perm_delete_with_obj_non_any_no_delete(self):
        permission_logic = CollaboratorsPermissionLogic(any_permission=False,
                                                 delete_permission=False)
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertFalse(
                permission_logic.has_perm(self.user2, self.perm3, self.article))



from django.test import TestCase, override_settings
from ...utils.handlers import PermissionHandlerRegistry
from ...handlers import PermissionHandler
from ..compat import MagicMock


@override_settings(
    PERMISSION_DEFAULT_PERMISSION_HANDLER=PermissionHandler
)
class PermissionUtilsHandlersTestCase(TestCase):
    def setUp(self):
        self.registry = PermissionHandlerRegistry()
        self.model = MagicMock()
        self.model._meta = MagicMock()
        self.model._meta.abstract = False
        self.handler = PermissionHandler

    def test_register(self):
        self.registry.register(self.model, self.handler)
        self.assertTrue(self.model in self.registry._registry)
        self.assertTrue(isinstance(self.registry._registry[self.model],
                                   self.handler))

    def test_register_without_specifing_handler(self):
        self.registry.register(self.model)
        self.assertTrue(self.model in self.registry._registry)
        self.assertTrue(isinstance(self.registry._registry[self.model],
                                   self.handler))

    def test_register_with_abstract_model(self):
        from django.core.exceptions import ImproperlyConfigured
        abstract_model = MagicMock()
        abstract_model._meta = MagicMock()
        abstract_model._meta.abstract = True
        self.assertRaises(ImproperlyConfigured,
                          self.registry.register,
                          abstract_model, self.handler)

    def test_register_duplicate(self):
        self.registry.register(self.model, self.handler)
        self.assertRaises(KeyError,
                          self.registry.register,
                          self.model, self.handler)

    def test_register_permission_handler_instance(self):
        handler_instance = self.handler(self.model)
        self.assertRaises(AttributeError,
                          self.registry.register,
                          self.model, handler_instance)

    def test_register_non_permission_handler(self):
        self.assertRaises(AttributeError,
                          self.registry.register,
                          self.model, self.__class__)

    def test_unregister(self):
        self.registry.register(self.model, self.handler)
        self.registry.unregister(self.model)
        self.assertFalse(self.model in self.registry._registry)

    def test_unregister_absence(self):
        self.assertRaises(KeyError,
                          self.registry.unregister,
                          self.model)

    def test_get_handlers(self):
        results = self.registry.get_handlers()
        self.assertTrue(isinstance(results, tuple))
        self.assertTrue(len(results) == 0)

        self.registry.register(self.model, self.handler)
        results = self.registry.get_handlers()
        self.assertTrue(isinstance(results, tuple))
        self.assertTrue(len(results) == 1)
        self.assertTrue(isinstance(results[0], PermissionHandler))




# coding=utf-8
"""
Permission logic module for collaborators based permission system
"""
from permission.conf import settings
from permission.logics.base import PermissionLogic
from permission.utils.field_lookup import field_lookup
from permission.compat import is_authenticated


class CollaboratorsPermissionLogic(PermissionLogic):
    """
    Permission logic class for collaborators based permission system
    """
    def __init__(self,
                 field_name=None,
                 any_permission=None,
                 change_permission=None,
                 delete_permission=None):
        """
        Constructor

        Parameters
        ----------
        field_name : string
            A field name of object which store the collaborators as django
            relational fields for django user model.
            You can specify the related object with '__' like django queryset
            filter.
            Default value will be taken from
            ``PERMISSION_DEFAULT_COLLABORATORS_PERMISSION_LOGIC_FIELD_NAME`` in
            settings.
        any_permission : boolean
            True for give any permission of the specified object to the
            collaborators.
            Default value will be taken from
            ``PERMISSION_DEFAULT_COLLABORATORS_PERMISSION_LOGIC_ANY_PERMISSION``
            in settings.
        change_permission : boolean
            True for give change permission of the specified object to the
            collaborators.
            It will be ignored if :attr:`any_permission` is True.
            Default value will be taken from
            ``PERMISSION_DEFAULT_COLLABORATORS_PERMISSION_LOGIC_CHANGE_PERMISSION``
            in settings.
        delete_permission : boolean
            True for give delete permission of the specified object to the
            collaborators.
            It will be ignored if :attr:`any_permission` is True.
            Default value will be taken from
            ``PERMISSION_DEFAULT_COLLABORATORS_PERMISSION_LOGIC_DELETE_PERMISSION``
            in settings.
        """
        self.field_name = field_name
        self.any_permission = any_permission
        self.change_permission = change_permission
        self.delete_permission = delete_permission

        if self.field_name is None:
            self.field_name = \
                settings.PERMISSION_DEFAULT_CPL_FIELD_NAME
        if self.any_permission is None:
            self.any_permission = \
                settings.PERMISSION_DEFAULT_CPL_ANY_PERMISSION
        if self.change_permission is None:
            self.change_permission = \
                settings.PERMISSION_DEFAULT_CPL_CHANGE_PERMISSION
        if self.delete_permission is None:
            self.delete_permission = \
                settings.PERMISSION_DEFAULT_CPL_DELETE_PERMISSION

    def has_perm(self, user_obj, perm, obj=None):
        """
        Check if user have permission (of object)

        If the user_obj is not authenticated, it return ``False``.

        If no object is specified, it return ``True`` when the corresponding
        permission was specified to ``True`` (changed from v0.7.0).
        This behavior is based on the django system.
        https://code.djangoproject.com/wiki/RowLevelPermissions


        If an object is specified, it will return ``True`` if the user is
        found in ``field_name`` of the object (e.g. ``obj.collaborators``).
        So once the object store the user as a collaborator in
        ``field_name`` attribute (default: ``collaborators``), the collaborator
        can change or delete the object (you can change this behavior to set
        ``any_permission``, ``change_permission`` or ``delete_permission``
        attributes of this instance).

        Parameters
        ----------
        user_obj : django user model instance
            A django user model instance which be checked
        perm : string
            `app_label.codename` formatted permission string
        obj : None or django model instance
            None or django model instance for object permission

        Returns
        -------
        boolean
            Whether the specified user have specified permission (of specified
            object).
        """
        if not is_authenticated(user_obj):
            return False
        # construct the permission full name
        change_permission = self.get_full_permission_string('change')
        delete_permission = self.get_full_permission_string('delete')
        if obj is None:
            # object permission without obj should return True
            # Ref: https://code.djangoproject.com/wiki/RowLevelPermissions
            if self.any_permission:
                return True
            if self.change_permission and perm == change_permission:
                return True
            if self.delete_permission and perm == delete_permission:
                return True
            return False
        elif user_obj.is_active:
            # get collaborator queryset
            collaborators = field_lookup(obj, self.field_name)
            if hasattr(collaborators, 'all'):
                collaborators = collaborators.all()
            if user_obj in collaborators:
                if self.any_permission:
                    # have any kind of permissions to the obj
                    return True
                if (self.change_permission and
                        perm == change_permission):
                    return True
                if (self.delete_permission and
                        perm == delete_permission):
                    return True
        return False



from __future__ import unicode_literals
from django.db import models
from django.conf import settings
from django.utils.encoding import python_2_unicode_compatible


AUTH_USER = getattr(settings, 'AUTH_USER_MODEL', 'auth.User')


@python_2_unicode_compatible
class Article(models.Model):
    title = models.CharField('title', max_length=200)
    content = models.TextField('content')

    author = models.ForeignKey(
        AUTH_USER, null=True,
        related_name='permission_test_articles_author')
    editor = models.ForeignKey(
        AUTH_USER, null=True,
        related_name='permission_test_articles_editor')
    authors = models.ManyToManyField(
        AUTH_USER, related_name='permission_test_articles_authors')
    editors = models.ManyToManyField(
        AUTH_USER, related_name='permission_test_articles_editors')

    single_bridge = models.ForeignKey(
        'permission.Bridge', null=True,
        related_name='permission_test_single_bridge')
    multiple_bridge = models.ManyToManyField(
        'permission.Bridge',
        related_name='permission_test_multiple_bridge')

    created_at = models.DateTimeField('created_at', auto_now_add=True)

    class Meta:
        app_label = 'permission'

    def __str__(self):
        return self.title


class Bridge(models.Model):
    author = models.ForeignKey(
        AUTH_USER, null=True, related_name='permission_test_bridge_author')
    editors = models.ManyToManyField(
        AUTH_USER, related_name='permission_test_bridge_editors')

    class Meta:
        app_label = 'permission'



# coding=utf-8
from django.db.models import Model
from django.http import HttpRequest
from django.http import HttpResponse
from permission.handlers import PermissionHandler
from permission.tests.compat import MagicMock


def create_mock_class(name, base, instance=None):
    instance = instance or MagicMock()
    mock_class = MagicMock(name=name,
                           return_value=instance)
    mock_class.__bases__ = (type, base)
    mock_class.__class__ = type
    return mock_class

def create_mock_handler():
    instance = MagicMock(**{
            'has_perm.return_value': False,
            'get_permissions.return_value': [
                'permission.add_article',
            ],
        })
    handler = create_mock_class('MockPermissionHandler',
                                base=PermissionHandler,
                                instance=instance)
    return handler


def create_mock_request(mock_permission_handler):
    request = MagicMock(spec=HttpRequest)
    request.build_absolute_uri = MagicMock(return_value="/")
    request.META = MagicMock()
    request.user = MagicMock(**{
            'is_active.return_value': True,
            'is_authenticated.return_value': True,
            'has_perm.side_effect': mock_permission_handler.has_perm,
        })
    return request


def create_mock_view_func():
    response = MagicMock(spec=HttpResponse)
    function = MagicMock(return_value=response)
    return function


def create_mock_view_class(view_func):
    from django.views.generic import View
    view_class = type('MockView', (View,), {})
    view_class.dispatch = view_func
    return view_class


def create_mock_model():
    instance = MagicMock()
    model = MagicMock(name='MockModel')
    model.__bases__ = (type,)
    model.__class__ = type
    model._meta = MagicMock(**{
            'abstract': False,
        })
    return model


def create_mock_queryset(obj):
    from django.db.models.query import QuerySet
    from django.core.exceptions import ObjectDoesNotExist
    def get_side_effect(*args, **kwargs):
        if kwargs.get('pk', None) == 1:
            return obj
        if kwargs.get('title', None) == 'permission_test_article1':
            return obj
        if kwargs.get('title__exact', None) == 'permission_test_article1':
            return obj
        raise queryset.model.DoesNotEixst
    queryset = MagicMock(spec=QuerySet, **{
            'get.side_effect': get_side_effect,
        })
    queryset.model = MagicMock()
    queryset.model.DoesNotEixst = ObjectDoesNotExist
    queryset.model._meta = MagicMock(object_name='MockQuerysetModel')
    return queryset



# coding=utf-8
from django.test import TestCase, override_settings
from ...logics import StaffPermissionLogic
from ...utils.logics import add_permission_logic
from ..utils import create_user, create_anonymous, create_article


@override_settings(
    PERMISSION_DEFAULT_SPL_ANY_PERMISSION=True,
    PERMISSION_DEFAULT_SPL_ADD_PERMISSION=True,
    PERMISSION_DEFAULT_SPL_CHANGE_PERMISSION=True,
    PERMISSION_DEFAULT_SPL_DELETE_PERMISSION=True,
)
class PermissionLogicsStaffPermissionLogicTestCase(TestCase):
    def setUp(self):
        self.user1 = create_user('john', is_staff=True)
        self.user2 = create_user('tony')
        self.anonymous = create_anonymous()
        self.perm1 = 'permission.add_article'
        self.perm2 = 'permission.change_article'
        self.perm3 = 'permission.delete_article'
        self.article = create_article('test')

    def test_constructor(self):
        permission_logic = StaffPermissionLogic()
        add_permission_logic(self.article.__class__, permission_logic)

        self.assertTrue(isinstance(permission_logic, StaffPermissionLogic))
        self.assertEqual(permission_logic.any_permission, True)
        self.assertEqual(permission_logic.add_permission, True)
        self.assertEqual(permission_logic.change_permission, True)
        self.assertEqual(permission_logic.delete_permission, True)

    def test_constructor_with_specifing_any_permission(self):
        permission_logic = StaffPermissionLogic(any_permission=False)
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertTrue(isinstance(permission_logic, StaffPermissionLogic))
        self.assertEqual(permission_logic.any_permission, False)

    def test_constructor_with_specifing_add_permission(self):
        permission_logic = StaffPermissionLogic(add_permission=False)
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertTrue(isinstance(permission_logic, StaffPermissionLogic))
        self.assertEqual(permission_logic.add_permission, False)

    def test_constructor_with_specifing_change_permission(self):
        permission_logic = StaffPermissionLogic(change_permission=False)
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertTrue(isinstance(permission_logic, StaffPermissionLogic))
        self.assertEqual(permission_logic.change_permission, False)

    def test_constructor_with_specifing_delete_permission(self):
        permission_logic = StaffPermissionLogic(delete_permission=False)
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertTrue(isinstance(permission_logic, StaffPermissionLogic))
        self.assertEqual(permission_logic.delete_permission, False)

    def test_has_perm_add_without_obj(self):
        permission_logic = StaffPermissionLogic()
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertTrue(permission_logic.has_perm(self.user1, self.perm1))
        self.assertFalse(permission_logic.has_perm(self.user2, self.perm1))

    def test_has_perm_change_without_obj(self):
        permission_logic = StaffPermissionLogic()
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertTrue(permission_logic.has_perm(self.user1, self.perm2))
        self.assertFalse(permission_logic.has_perm(self.user2, self.perm2))

    def test_has_perm_delete_without_obj(self):
        permission_logic = StaffPermissionLogic()
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertTrue(permission_logic.has_perm(self.user1, self.perm3))
        self.assertFalse(permission_logic.has_perm(self.user2, self.perm3))

    def test_has_perm_add_with_obj(self):
        permission_logic = StaffPermissionLogic()
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertTrue(
                permission_logic.has_perm(self.user1, self.perm1, self.article))
        self.assertFalse(
                permission_logic.has_perm(self.user2, self.perm1, self.article))

    def test_has_perm_change_with_obj(self):
        permission_logic = StaffPermissionLogic()
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertTrue(
                permission_logic.has_perm(self.user1, self.perm2, self.article))
        self.assertFalse(
                permission_logic.has_perm(self.user2, self.perm2, self.article))

    def test_has_perm_delete_with_obj(self):
        permission_logic = StaffPermissionLogic()
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertTrue(
                permission_logic.has_perm(self.user1, self.perm3, self.article))
        self.assertFalse(
                permission_logic.has_perm(self.user2, self.perm3, self.article))

    def test_has_perm_add_without_obj_with_anonymous(self):
        permission_logic = StaffPermissionLogic()
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertFalse(permission_logic.has_perm(self.anonymous, self.perm1))

    def test_has_perm_change_without_obj_with_anonymous(self):
        permission_logic = StaffPermissionLogic()
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertFalse(permission_logic.has_perm(self.anonymous, self.perm2))

    def test_has_perm_delete_without_obj_with_anonymous(self):
        permission_logic = StaffPermissionLogic()
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertFalse(permission_logic.has_perm(self.anonymous, self.perm3))

    def test_has_perm_add_with_obj_with_anonymous(self):
        permission_logic = StaffPermissionLogic()
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertFalse(
            permission_logic.has_perm(self.anonymous, self.perm1, self.article))

    def test_has_perm_change_with_obj_with_anonymous(self):
        permission_logic = StaffPermissionLogic()
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertFalse(
            permission_logic.has_perm(self.anonymous, self.perm2, self.article))

    def test_has_perm_delete_with_obj_with_anonymous(self):
        permission_logic = StaffPermissionLogic()
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertFalse(
            permission_logic.has_perm(self.anonymous, self.perm3, self.article))

    def test_has_perm_add_without_obj_without_any(self):
        permission_logic = StaffPermissionLogic(any_permission=False)
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertTrue(permission_logic.has_perm(self.user1, self.perm1))
        self.assertFalse(permission_logic.has_perm(self.user2, self.perm1))

    def test_has_perm_change_without_obj_without_any(self):
        permission_logic = StaffPermissionLogic(any_permission=False)
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertTrue(permission_logic.has_perm(self.user1, self.perm2))
        self.assertFalse(permission_logic.has_perm(self.user2, self.perm2))

    def test_has_perm_delete_without_obj_without_any(self):
        permission_logic = StaffPermissionLogic(any_permission=False)
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertTrue(permission_logic.has_perm(self.user1, self.perm3))
        self.assertFalse(permission_logic.has_perm(self.user2, self.perm3))

    def test_has_perm_add_with_obj_without_any(self):
        permission_logic = StaffPermissionLogic(any_permission=False)
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertTrue(
                permission_logic.has_perm(self.user1, self.perm1, self.article))
        self.assertFalse(
                permission_logic.has_perm(self.user2, self.perm1, self.article))

    def test_has_perm_change_with_obj_without_any(self):
        permission_logic = StaffPermissionLogic(any_permission=False)
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertTrue(
                permission_logic.has_perm(self.user1, self.perm2, self.article))
        self.assertFalse(
                permission_logic.has_perm(self.user2, self.perm2, self.article))

    def test_has_perm_delete_with_obj_without_any(self):
        permission_logic = StaffPermissionLogic(any_permission=False)
        add_permission_logic(self.article.__class__, permission_logic)
        self.assertTrue(
                permission_logic.has_perm(self.user1, self.perm3, self.article))
        self.assertFalse(
                permission_logic.has_perm(self.user2, self.perm3, self.article))



# coding=utf-8
"""
django-permission application configure
"""
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from appconf import AppConf
from permission.handlers import LogicalPermissionHandler


__all__ = ('settings',)


class PermissionConf(AppConf):
    DEFAULT_PERMISSION_HANDLER = 'permission.handlers.LogicalPermissionHandler'
    """Default permission handler class"""

    CHECK_PERMISSION_PRESENCE = settings.DEBUG
    """Check if the specified string permission exists"""

    REPLACE_BUILTIN_IF = True
    """Whether replace builtin if templatetag"""

    DEFAULT_APL_FIELD_NAME = 'author'
    DEFAULT_APL_ANY_PERMISSION = False
    DEFAULT_APL_CHANGE_PERMISSION = True
    DEFAULT_APL_DELETE_PERMISSION = True

    DEFAULT_CPL_FIELD_NAME = 'collaborators'
    DEFAULT_CPL_ANY_PERMISSION = False
    DEFAULT_CPL_CHANGE_PERMISSION = True
    DEFAULT_CPL_DELETE_PERMISSION = False

    DEFAULT_GIPL_ANY_PERMISSION = False
    DEFAULT_GIPL_ADD_PERMISSION = True
    DEFAULT_GIPL_CHANGE_PERMISSION = True
    DEFAULT_GIPL_DELETE_PERMISSION = False

    DEFAULT_OSPL_ANY_PERMISSION = False
    DEFAULT_OSPL_CHANGE_PERMISSION = True
    DEFAULT_OSPL_DELETE_PERMISSION = True

    DEFAULT_SPL_ANY_PERMISSION = False
    DEFAULT_SPL_ADD_PERMISSION = True
    DEFAULT_SPL_CHANGE_PERMISSION = True
    DEFAULT_SPL_DELETE_PERMISSION = False

    AUTODISCOVER_MODULE_NAME = 'perms'
    AUTODISCOVER_VARIABLE_NAME = 'PERMISSION_LOGICS'
    AUTODISCOVER_ENABLE = True

    CHECK_AUTHENTICATION_BACKENDS = True
    """Check if AUTHENTICATION_BACKENDS is correctly configured"""

    CHECK_TEMPLATES_OPTIONS_BUILTINS = True
    """Check if TEMPLATES[?]['OPTIONS']['builtins'] is correctly configured"""



# coding=utf-8
from django.test import TestCase, override_settings
from ... import add_permission_logic
from ...logics import OneselfPermissionLogic
from ..utils import create_user, create_anonymous, create_article
from ..compat import MagicMock


@override_settings(
    PERMISSION_DEFAULT_OSPL_ANY_PERMISSION=True,
    PERMISSION_DEFAULT_OSPL_CHANGE_PERMISSION=True,
    PERMISSION_DEFAULT_OSPL_DELETE_PERMISSION=True,
)
class PermissionLogicsOneselfPermissionLogicTestCase(TestCase):
    def setUp(self):
        self.user1 = create_user('john')
        self.user2 = create_user('tony')
        self.anonymous = create_anonymous()
        self.perm1 = 'auth.add_user'
        self.perm2 = 'auth.change_user'
        self.perm3 = 'auth.delete_user'

    def test_constructor(self):
        permission_logic = OneselfPermissionLogic()
        add_permission_logic(self.user1.__class__, permission_logic)
        self.assertTrue(isinstance(permission_logic, OneselfPermissionLogic))
        self.assertEqual(permission_logic.any_permission, True)
        self.assertEqual(permission_logic.change_permission, True)
        self.assertEqual(permission_logic.delete_permission, True)

    def test_constructor_with_specifying_any_permission(self):
        permission_logic = OneselfPermissionLogic(any_permission=False)
        add_permission_logic(self.user1.__class__, permission_logic)
        self.assertTrue(isinstance(permission_logic, OneselfPermissionLogic))
        self.assertEqual(permission_logic.any_permission, False)

    def test_constructor_with_specifying_change_permission(self):
        permission_logic = OneselfPermissionLogic(change_permission=False)
        add_permission_logic(self.user1.__class__, permission_logic)
        self.assertTrue(isinstance(permission_logic, OneselfPermissionLogic))
        self.assertEqual(permission_logic.change_permission, False)

    def test_constructor_with_specifying_delete_permission(self):
        permission_logic = OneselfPermissionLogic(delete_permission=False)
        add_permission_logic(self.user1.__class__, permission_logic)
        self.assertTrue(isinstance(permission_logic, OneselfPermissionLogic))
        self.assertEqual(permission_logic.delete_permission, False)

    def test_has_perm_add_without_obj(self):
        permission_logic = OneselfPermissionLogic()
        add_permission_logic(self.user1.__class__, permission_logic)
        self.assertTrue(permission_logic.has_perm(self.user1, self.perm1))

    def test_has_perm_change_without_obj(self):
        permission_logic = OneselfPermissionLogic()
        add_permission_logic(self.user1.__class__, permission_logic)
        self.assertTrue(permission_logic.has_perm(self.user1, self.perm2))

    def test_has_perm_delete_without_obj(self):
        permission_logic = OneselfPermissionLogic()
        add_permission_logic(self.user1.__class__, permission_logic)
        self.assertTrue(permission_logic.has_perm(self.user1, self.perm3))

    def test_has_perm_add_without_obj_with_anonymous(self):
        permission_logic = OneselfPermissionLogic()
        add_permission_logic(self.user1.__class__, permission_logic)
        self.assertFalse(permission_logic.has_perm(self.anonymous, self.perm1))

    def test_has_perm_change_without_obj_with_anonymous(self):
        permission_logic = OneselfPermissionLogic()
        add_permission_logic(self.user1.__class__, permission_logic)
        self.assertFalse(permission_logic.has_perm(self.anonymous, self.perm2))

    def test_has_perm_delete_without_obj_with_anonymous(self):
        permission_logic = OneselfPermissionLogic()
        add_permission_logic(self.user1.__class__, permission_logic)
        self.assertFalse(permission_logic.has_perm(self.anonymous, self.perm3))

    def test_has_perm_add_with_obj(self):
        permission_logic = OneselfPermissionLogic()
        add_permission_logic(self.user1.__class__, permission_logic)
        self.assertFalse(
                permission_logic.has_perm(self.user1, self.perm1, self.user2))

    def test_has_perm_change_with_obj(self):
        permission_logic = OneselfPermissionLogic()
        add_permission_logic(self.user1.__class__, permission_logic)
        self.assertFalse(
                permission_logic.has_perm(self.user1, self.perm2, self.user2))

    def test_has_perm_delete_with_obj(self):
        permission_logic = OneselfPermissionLogic()
        add_permission_logic(self.user1.__class__, permission_logic)
        self.assertFalse(
                permission_logic.has_perm(self.user1, self.perm3, self.user2))

    def test_has_perm_add_with_obj_with_anonymous(self):
        permission_logic = OneselfPermissionLogic()
        add_permission_logic(self.user1.__class__, permission_logic)
        self.assertFalse(
            permission_logic.has_perm(self.anonymous, self.perm1, self.user1))

    def test_has_perm_change_with_obj_with_anonymous(self):
        permission_logic = OneselfPermissionLogic()
        add_permission_logic(self.user1.__class__, permission_logic)
        self.assertFalse(
            permission_logic.has_perm(self.anonymous, self.perm2, self.user1))

    def test_has_perm_delete_with_obj_with_anonymous(self):
        permission_logic = OneselfPermissionLogic()
        add_permission_logic(self.user1.__class__, permission_logic)
        self.assertFalse(
            permission_logic.has_perm(self.anonymous, self.perm3, self.user1))

    def test_has_perm_add_with_himself(self):
        permission_logic = OneselfPermissionLogic()
        add_permission_logic(self.user1.__class__, permission_logic)
        self.assertTrue(
                permission_logic.has_perm(self.user1, self.perm1, self.user1))

    def test_has_perm_change_with_himself(self):
        permission_logic = OneselfPermissionLogic()
        add_permission_logic(self.user1.__class__, permission_logic)
        self.assertTrue(
                permission_logic.has_perm(self.user1, self.perm2, self.user1))

    def test_has_perm_delete_with_himself(self):
        permission_logic = OneselfPermissionLogic()
        add_permission_logic(self.user1.__class__, permission_logic)
        self.assertTrue(
                permission_logic.has_perm(self.user1, self.perm3, self.user1))

    def test_has_perm_add_with_himself_non_any(self):
        permission_logic = OneselfPermissionLogic(any_permission=False)
        add_permission_logic(self.user1.__class__, permission_logic)
        self.assertFalse(
                permission_logic.has_perm(self.user1, self.perm1, self.user1))

    def test_has_perm_change_with_himself_non_any(self):
        permission_logic = OneselfPermissionLogic(any_permission=False)
        add_permission_logic(self.user1.__class__, permission_logic)
        self.assertTrue(
                permission_logic.has_perm(self.user1, self.perm2, self.user1))

    def test_has_perm_delete_with_obj_non_any(self):
        permission_logic = OneselfPermissionLogic(any_permission=False)
        add_permission_logic(self.user1.__class__, permission_logic)
        self.assertTrue(
                permission_logic.has_perm(self.user1, self.perm3, self.user1))

    def test_has_perm_add_with_himself_non_any_no_change(self):
        permission_logic = OneselfPermissionLogic(any_permission=False,
                                                 change_permission=False)
        add_permission_logic(self.user1.__class__, permission_logic)
        self.assertFalse(
                permission_logic.has_perm(self.user1, self.perm1, self.user1))

    def test_has_perm_change_with_himself_non_any_no_change(self):
        permission_logic = OneselfPermissionLogic(any_permission=False,
                                                 change_permission=False)
        add_permission_logic(self.user1.__class__, permission_logic)
        self.assertFalse(
                permission_logic.has_perm(self.user1, self.perm2, self.user1))

    def test_has_perm_delete_with_himself_non_any_no_change(self):
        permission_logic = OneselfPermissionLogic(any_permission=False,
                                                 change_permission=False)
        add_permission_logic(self.user1.__class__, permission_logic)
        self.assertTrue(
                permission_logic.has_perm(self.user1, self.perm3, self.user1))

    def test_has_perm_add_with_himself_non_any_no_delete(self):
        permission_logic = OneselfPermissionLogic(any_permission=False,
                                                 delete_permission=False)
        add_permission_logic(self.user1.__class__, permission_logic)
        self.assertFalse(
                permission_logic.has_perm(self.user1, self.perm1, self.user1))

    def test_has_perm_change_with_himself_non_any_no_delete(self):
        permission_logic = OneselfPermissionLogic(any_permission=False,
                                                 delete_permission=False)
        add_permission_logic(self.user1.__class__, permission_logic)
        self.assertTrue(
                permission_logic.has_perm(self.user1, self.perm2, self.user1))

    def test_has_perm_delete_with_himself_non_any_no_delete(self):
        permission_logic = OneselfPermissionLogic(any_permission=False,
                                                 delete_permission=False)
        add_permission_logic(self.user1.__class__, permission_logic)
        self.assertFalse(
                permission_logic.has_perm(self.user1, self.perm3, self.user1))



# coding=utf-8


class PermissionLogic(object):
    """
    Abstract permission logic class
    """
    def get_full_permission_string(self, perm):
        """
        Return full permission string (app_label.perm_model)
        """
        if not getattr(self, 'model', None):
            raise AttributeError("You need to use `add_permission_logic` to "
                                 "register the instance to the model class "
                                 "before calling this method.")
        app_label = self.model._meta.app_label
        model_name = self.model._meta.object_name.lower()
        return "%s.%s_%s" % (app_label, perm, model_name)

    def has_perm(self, user_obj, perm, obj=None):
        """
        Check if user have permission (of object)

        Parameters
        ----------
        user_obj : django user model instance
            A django user model instance which be checked
        perm : string
            `app_label.codename` formatted permission string
        obj : None or django model instance
            None or django model instance for object permission

        Returns
        -------
        boolean
            Whether the specified user have specified permission (of specified
            object).

        .. note::
            Sub class must override this method.
        """
        raise NotImplementedError(
                "'%s' does not override `has_perm(user_obj, perm, obj=None)` "
                "method. Sub class of `PermissionLogic` must override this "
                "method.")



# coding=utf-8
import django
if django.VERSION >= (1, 9):
    add_to_builtins = None
else:
    try:
        from django.template.base import add_to_builtins
    except ImportError:
        from django.template.loader import add_to_builtins

try:
    # django.utils.importlib is removed from Django 1.9
    from importlib import import_module
except ImportError:
    from django.utils.importlib import import_module

if django.VERSION < (1, 10):
    def is_authenticated(user_obj):
        return user_obj.is_authenticated()

    def is_anonyomus(user_obj):
        return user_obj.is_anonymous()
else:
    def is_authenticated(user_obj):
        return user_obj.is_authenticated

    def is_anonyomus(user_obj):
        return user_obj.is_anonymous
 
try:
    # Django 1.7 or over use the new application loading system
    from django.apps import apps
    get_model = apps.get_model
except ImportError:
    from django.db.models.loading import get_model

try:
    from django.utils.module_loading import import_string
except ImportError:
    try:
        from django.utils.module_loading import import_by_path as import_string
    except ImportError:
        def import_string(dotted_path):
            try:
                module_path, class_name = dotted_path.rsplit('.', 1)
            except ValueError:
                raise ImportError(
                    "%s doesn't look like a module path" % dotted_path
                )
            module = import_module(module_path)
            try:
                return getattr(module, class_name)
            except AttributeError:
                raise ImportError(
                    'Module "%s" does not define a "%s" attribute/class' % (
                        module_path, class_name
                    ))

try:
    # Python 3
    from urllib.parse import urlparse
except ImportError:
    # Python 2
    from urlparse import urlparse

import sys
if sys.version_info >= (3, 0):
    def isstr(x):
        return isinstance(x, str)
else:
    def isstr(x):
        return isinstance(x, basestring)

try:
    from django.util import six
except ImportError:
    # Django 1.2/1.3 does not have six
    import six



# coding=utf-8
from django.test import TestCase
from ...utils.field_lookup import field_lookup
from ..utils import create_user, create_bridge, create_article


class PermissionUtilsFieldLookupTestCase(TestCase):
    def setUp(self):
        self.author = create_user('permission_test_articles_author')
        self.editor1 = create_user(
            'permission_test_articles_editor1')
        self.editor2 = create_user(
            'permission_test_articles_editor2')
        self.bridge1 = create_bridge()
        self.bridge2 = create_bridge()
        self.bridge3 = create_bridge()
        self.model = create_article('permission_test_article',
                                    self.author, self.bridge1)
        self.model.editors.add(self.editor1)
        self.model.editors.add(self.editor2)
        self.model.multiple_bridge.add(self.bridge2)
        self.model.multiple_bridge.add(self.bridge3)

    def test_field_lookup_author(self):
        field_value = field_lookup(self.model, 'author')
        self.assertEqual(field_value, self.author)

    def test_field_lookup_author_username(self):
        field_value = field_lookup(self.model, 'author__username')
        self.assertEqual(field_value, self.author.username)

    def test_field_lookup_editors(self):
        field_value = field_lookup(self.model, 'editors')
        field_value = (x for x in field_value.iterator())
        expected_value = list(map(repr, (self.editor1, self.editor2)))
        self.assertQuerysetEqual(field_value, expected_value)

    def test_field_lookup_editors_username(self):
        field_value = list(field_lookup(self.model, 'editors__username'))
        expected_value = [x.username for x in
                          (self.editor1, self.editor2)]
        self.assertEqual(field_value, expected_value)

    def test_field_lookup_single_bridge_author(self):
        field_value = field_lookup(self.model, 'single_bridge__author')
        self.assertEqual(field_value, self.bridge1.author)

    def test_field_lookup_single_bridge_author_username(self):
        field_value = field_lookup(self.model,
                                   'single_bridge__author__username')
        self.assertEqual(field_value, self.bridge1.author.username)

    def test_field_lookup_single_bridge_editors(self):
        field_value = field_lookup(self.model, 'single_bridge__editors')
        field_value = (x for x in field_value.iterator())
        expected_value = list(map(repr, self.bridge1.editors.iterator()))
        self.assertQuerysetEqual(field_value, expected_value)

    def test_field_lookup_single_bridge_editors_username(self):
        field_value = list(field_lookup(self.model,
                                        'single_bridge__editors__username'))
        expected_value = [x.username for x in self.bridge1.editors.iterator()]
        self.assertEqual(field_value, expected_value)

    def test_field_lookup_multiple_bridge_author(self):
        field_value = field_lookup(self.model, 'multiple_bridge__author')
        expected_value = list(map(repr, (self.bridge2.author,
                                         self.bridge3.author)))
        self.assertQuerysetEqual(field_value, expected_value)

    def test_field_lookup_multiple_bridge_author_username(self):
        field_value = field_lookup(self.model,
                                   'multiple_bridge__author__username')
        field_value = list(field_value)
        expected_value = [x.username for x in (self.bridge2.author,
                                               self.bridge3.author)]
        self.assertEqual(field_value, expected_value)

    def test_field_lookup_multiple_bridge_editors(self):
        field_value = list(field_lookup(self.model,
                                        'multiple_bridge__editors'))
        field_value = [
            [repr(x) for x in field_value[0].iterator()],
            [repr(x) for x in field_value[1].iterator()],
        ]
        expected_value1 = [repr(x) for x in self.bridge2.editors.iterator()]
        expected_value2 = [repr(x) for x in self.bridge3.editors.iterator()]
        expected_value = [expected_value1, expected_value2]
        self.assertEqual(field_value, expected_value)

    def test_field_lookup_multiple_bridge_editors__name(self):
        field_value = field_lookup(self.model,
                                   'multiple_bridge__editors__username')
        field_value = list(map(list, field_value))
        expected_value1 = [x.username for x in self.bridge2.editors.iterator()]
        expected_value2 = [x.username for x in self.bridge3.editors.iterator()]
        expected_value = [expected_value1, expected_value2]
        self.assertEqual(field_value, expected_value)




# coding=utf-8


def create_user(username, **kwargs):
    from django.contrib.auth.models import User
    user = User.objects.create_user(
        username=username,
        email="%s@test.com" % username,
        password="password",
    )
    # attribute assignment
    for key, value in kwargs.items():
        user.__dict__[key] = value
    user.save()
    return user


def create_anonymous(**kwargs):
    from django.contrib.auth.models import AnonymousUser
    return AnonymousUser(**kwargs)


def create_group(name, user=None):
    from django.contrib.auth.models import Group
    group = Group.objects.create(name=name)
    if user is not None:
        user.groups.add(group)
        user.save()
    group.save()
    return group


def create_article(title, user=None, bridge=None):
    import datetime
    from permission.tests.models import Article
    user = user or create_user(str(datetime.datetime.now()))
    article = Article.objects.create(
        title=title,
        content=title*20,
        author=user,
        single_bridge=bridge
    )
    article.save()
    return article


def create_bridge(user=None, editors=None):
    import datetime
    from permission.tests.models import Bridge
    user = user or create_user(str(datetime.datetime.now()))
    editors = editors or [create_user(str(datetime.datetime.now())+str(i))
                          for i in range(2)]
    bridge = Bridge.objects.create(author=user)
    for editor in editors:
        bridge.editors.add(editor)
    bridge.save()
    return bridge


def create_permission(name, model=None):
    from django.contrib.auth.models import Permission
    from django.contrib.contenttypes.models import ContentType
    from permission.tests.models import Article
    model = model or Article
    ct = ContentType.objects.get_for_model(model)
    permission = Permission.objects.create(
        name=name, codename=name,
        content_type=ct
    )
    permission.save()
    return permission



from django.apps import AppConfig


class PermissionConfig(AppConfig):
    name = "permission"
    verbose_name = "Permission"

    def ready(self):
        from permission.conf import settings
        if settings.PERMISSION_AUTODISCOVER_ENABLE:
            from permission.utils.autodiscover import autodiscover
            autodiscover()



# coding=utf-8
"""
Logical permission backends module
"""
from permission.conf import settings
from permission.utils.handlers import registry
from permission.utils.permissions import perm_to_permission


__all__    = ('PermissionBackend',)


class PermissionBackend(object):
    """
    A handler based permission backend
    """
    supports_object_permissions = True
    supports_anonymous_user = True
    supports_inactive_user = True

    # pylint:disable=unused-argument
    def authenticate(self, username, password):
        """
        Always return ``None`` to prevent authentication within this backend.
        """
        return None

    def has_perm(self, user_obj, perm, obj=None):
        """
        Check if user have permission (of object) based on registered handlers.

        It will raise ``ObjectDoesNotExist`` exception when the specified
        string permission does not exist and
        ``PERMISSION_CHECK_PERMISSION_PRESENCE`` is ``True`` in ``settings``
        module.

        Parameters
        ----------
        user_obj : django user model instance
            A django user model instance which be checked
        perm : string
            `app_label.codename` formatted permission string
        obj : None or django model instance
            None or django model instance for object permission

        Returns
        -------
        boolean
            Whether the specified user have specified permission (of specified
            object).

        Raises
        ------
        django.core.exceptions.ObjectDoesNotExist
            If the specified string permission does not exist and
            ``PERMISSION_CHECK_PERMISSION_PRESENCE`` is ``True`` in ``settings``
            module.
        """
        if settings.PERMISSION_CHECK_PERMISSION_PRESENCE:
            # get permission instance from string permission (perm)
            # it raise ObjectDoesNotExists when the permission is not exists
            try:
                perm_to_permission(perm)
            except AttributeError:
                # Django 1.2 internally use wrong permission string thus ignore
                pass

        # get permission handlers fot this perm
        cache_name = '_%s_cache' % perm
        if hasattr(self, cache_name):
            handlers = getattr(self, cache_name)
        else:
            handlers = [h for h in registry.get_handlers()
                        if perm in h.get_supported_permissions()]
            setattr(self, cache_name, handlers)
        for handler in handlers:
            if handler.has_perm(user_obj, perm, obj=obj):
                return True
        return False

    def has_module_perms(self, user_obj, app_label):
        """
        Check if user have permission of specified app based on registered
        handlers.

        It will raise ``ObjectDoesNotExist`` exception when the specified
        string permission does not exist and
        ``PERMISSION_CHECK_PERMISSION_PRESENCE`` is ``True`` in ``settings``
        module.

        Parameters
        ----------
        user_obj : django user model instance
            A django user model instance which is checked
        app_label : string
            `app_label.codename` formatted permission string

        Returns
        -------
        boolean
            Whether the specified user have specified permission.

        Raises
        ------
        django.core.exceptions.ObjectDoesNotExist
            If the specified string permission does not exist and
            ``PERMISSION_CHECK_PERMISSION_PRESENCE`` is ``True`` in ``settings``
            module.
        """
        # get permission handlers fot this perm
        cache_name = '_%s_cache' % app_label
        if hasattr(self, cache_name):
            handlers = getattr(self, cache_name)
        else:
            handlers = [h for h in registry.get_handlers()
                        if app_label in h.get_supported_app_labels()]
            setattr(self, cache_name, handlers)
        for handler in handlers:
            if handler.has_module_perms(user_obj, app_label):
                return True
        return False



# coding=utf-8
from django.test import TestCase
from ...logics import PermissionLogic
from ..utils import create_user, create_article


class PermissionLogicsPermissionLogicTestCase(TestCase):

    def setUp(self):
        self.user = create_user('john')
        self.perm1 = 'permission.add_article'
        self.perm2 = 'permission.change_article'
        self.perm3 = 'permission.delete_article'
        self.article = create_article('test')

    def test_constructor(self):
        permission_logic = PermissionLogic()
        self.assertTrue(isinstance(permission_logic, PermissionLogic))

    def test_has_perm_add_wihtout_obj(self):
        permission_logic = PermissionLogic()
        self.assertRaises(NotImplementedError,
                          permission_logic.has_perm,
                          self.user, self.perm1)

    def test_has_perm_change_wihtout_obj(self):
        permission_logic = PermissionLogic()
        self.assertRaises(NotImplementedError,
                          permission_logic.has_perm,
                          self.user, self.perm2)

    def test_has_perm_delete_wihtout_obj(self):
        permission_logic = PermissionLogic()
        self.assertRaises(NotImplementedError,
                          permission_logic.has_perm,
                          self.user, self.perm3)

    def test_has_perm_add_wiht_obj(self):
        permission_logic = PermissionLogic()
        self.assertRaises(NotImplementedError,
                          permission_logic.has_perm,
                          self.user, self.perm1, self.article)

    def test_has_perm_change_wiht_obj(self):
        permission_logic = PermissionLogic()
        self.assertRaises(NotImplementedError,
                          permission_logic.has_perm,
                          self.user, self.perm2, self.article)

    def test_has_perm_delete_wiht_obj(self):
        permission_logic = PermissionLogic()
        self.assertRaises(NotImplementedError,
                          permission_logic.has_perm,
                          self.user, self.perm3, self.article)
